/*
**  File: Amotry.c
**  Purpose: simplex operation function
**  Updated by: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: Amotry.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"

float Amotry(TRAIN_PROTOCOL *train_protocol, void *seg_param, void *ben_param,
	     char *weight_file, SIMPLEX_PARAM *simplex_param, float *psum,
	     void (*genscore)(),
	     int ihi, int *nFuncEval_p, float fac, FILE *fp)
{
   int j;
   float *ptry;
   float fac1, fac2;
   float ytry, ttry;

   SCORE *score;

   score = (SCORE *) malloc (sizeof(SCORE));
	

   ptry = (float *) malloc ( (simplex_param->ndim + 1)  * sizeof(float));
   fac1 = (1.0 - fac) / simplex_param->ndim;
   fac2 = fac1 - fac;
   for(j = 1; j <= simplex_param->ndim; j++) 
     ptry[j] = psum[j] * fac1 - simplex_param->p[ihi][j] * fac2;
   (*genscore)(train_protocol, seg_param, ben_param, weight_file, ptry, &score);
   ++(*nFuncEval_p);
   ytry = score->err_rate;
   ttry = score->timing;
   free(score);

   fprintf(fp,"%d\t", (*nFuncEval_p));	/* print result in report file */
   for(j = 1; j <= simplex_param->ndim; j++)
   fprintf(fp,"%.3f\t", ptry[j]);
   fprintf(fp,"%.3f\t", ytry * 100);
   fprintf(fp,"%.1f\t", ttry);

   if(ytry < simplex_param->y[ihi]) {
     simplex_param->y[ihi] = ytry;
     for(j = 1; j <= simplex_param->ndim; j++) {
       psum[j] += ptry[j] - simplex_param->p[ihi][j];
       simplex_param->p[ihi][j] = ptry[j];
     }
   }
   free(ptry);
   return ytry;
}
