/*  
**  File: Bench.c
**  Purpose: benchmarking algorithm function implementation
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   29, March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  References:
**  $Id: Bench.c,v 1.1 2000/09/19 22:16:27 maosong Exp maosong $
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>


#include <sys/types.h>
#include <sys/uio.h>
#include <sys/ddi.h>

#include "rafport.h"
#include "dafstype.h"
#include "rlimage.h"
#include "dafs.h"

#include "pset.h"

void Bench(char *img, BENCH_PARAM *para, char *gtdafs, char *sgdafs,
	   ERR_MEASURE **err_measure_p)
{
  	iEntityPtr gt_doc, sg_doc;
  	iDAFSError dafs_error;
  	TEXTLINE   **textline;
  	iBox       *gt_zone;
  	ZONE       *sg_zone; 
  	int        gt_zone_count=0;
  	int        sg_zone_count=0;
  	int        *textline_count;


  	ReadInput(gtdafs, sgdafs, &gt_doc, &sg_doc, &dafs_error);

  	ReadZoneInfo(gt_doc, sg_doc, &gt_zone_count, &sg_zone_count,
	 	      &textline_count, &textline, &gt_zone, &sg_zone);

  	CmpGt_Sg(para, gtdafs, textline, gt_zone, sg_zone, gt_zone_count,
	  	 sg_zone_count, textline_count, err_measure_p);

  	GetError(img, textline, gt_zone, sg_zone, gt_zone_count, sg_zone_count,
		   textline_count);

}


/* < Read in dafs files */
void ReadInput(char *gtdafs, char *sgdafs, iEntityPtr *gt_doc_p,
		iEntityPtr *sg_doc_p, iDAFSError *dafs_error_p) 
{
  	*dafs_error_p = i_InitLib();
  	*dafs_error_p = i_ReadEntity(gtdafs, gt_doc_p);
  	*dafs_error_p = i_ReadEntity(sgdafs, sg_doc_p);
}
/* > */

/* < Read groundtruth and segmented zones information */
void ReadZoneInfo(iEntityPtr gt_doc, iEntityPtr sg_doc,
		  int *gt_zone_count_p, int *sg_zone_count_p,
		  int **textline_count_p, TEXTLINE  ***textline_p,
		  iBox **gt_zone_p, ZONE **sg_zone_p)
{
  	int i;
  	GetZoneNo(gt_doc, gt_zone_count_p);
  	*textline_p = (TEXTLINE **) malloc ( (*gt_zone_count_p) * sizeof(TEXTLINE *));
  	*textline_count_p = (int *) malloc ( (*gt_zone_count_p) * sizeof(int));
  	for(i=0; i < (*gt_zone_count_p); i++) 
    		(*textline_count_p)[i]=0;
  	(*gt_zone_p) = (iBox *) malloc ( (*gt_zone_count_p) * sizeof(iBox) );
  	(*gt_zone_count_p) = 0;
  	GetTextlineInfo(gt_doc, gt_zone_count_p, gt_zone_p, textline_count_p, textline_p);
  	GetSGZoneNo(sg_doc, sg_zone_count_p);
  	(*sg_zone_p) = (ZONE *) malloc ((*sg_zone_count_p) * sizeof(ZONE));
  	(*sg_zone_count_p) = 0;
  	GetSGZoneInfo(sg_doc, sg_zone_count_p, sg_zone_p);

  	i_DisposeEntity(gt_doc);
  	i_DisposeEntity(sg_doc);
  	i_ExitLib();

}
/* > */


/* <  Obtain the number of ground truth zones */
void GetZoneNo(iEntityPtr root, int *gt_zone_count_p)
{
  	const char *ch="Zone";
  	iEntityPtr child;

  	if(strcmp(ch,i_TypeToName(i_GetEntityType(root))) == 0)
    		(*gt_zone_count_p)++;    

  	for(child= i_GetChildEntity(root); child; child=i_GetNextEntity(child))
    	GetZoneNo(child, gt_zone_count_p);
  
}
/* > */


/* < Obtain the number of Segmentation zones */
void GetSGZoneNo(iEntityPtr root, int *sg_zone_count_p)
{

 	const char *ch="Paragraph";
  	iEntityPtr child;

  	if(strcmp(ch,i_TypeToName(i_GetEntityType(root))) == 0) 
    		(*sg_zone_count_p)++;    

  	for(child= i_GetChildEntity(root); child; child=i_GetNextEntity(child))
    	GetSGZoneNo(child, sg_zone_count_p);
 
}
/* > */


/* < Load Textline information */
int GetTextlineInfo(iEntityPtr root, int *gt_zone_count_p, iBox **gt_zone_p,
		    int **textline_count_p, TEXTLINE ***textline_p)
{
  	const char *ch="Zone";
  	int i;
  	int p, c;
  	iEntityPtr child;

  	void *data;
  	iATag type;

  	if(strcmp(ch,i_TypeToName(i_GetEntityType(root))) == 0) {
    		i_GetArea(root, &data, &type);
    		switch(type) {
    			case iABox:
      				(*gt_zone_p)[(*gt_zone_count_p)] = *(iBox *)data;
      				break;
    			default:
      				break;
    		}
    		c = i_CountChildren(root);
    		if( c == 0 ) {
      			(*textline_p)[(*gt_zone_count_p)] =
				(TEXTLINE *)malloc(1*sizeof(TEXTLINE));
      			(*textline_p)[(*gt_zone_count_p)][0].sg_index = 0;
      			(*textline_p)[(*gt_zone_count_p)][0].textline_flg = NO;
      			(*textline_p)[(*gt_zone_count_p)][0].box =
				(*gt_zone_p)[(*gt_zone_count_p)];
      			(*textline_p)[(*gt_zone_count_p)][0].miss_det = NO;
      			(*textline_p)[(*gt_zone_count_p)][0].h_split = NO;
      			(*textline_p)[(*gt_zone_count_p)][0].h_merge = NO;
      			(*textline_p)[(*gt_zone_count_p)][0].v_split = NO;
      			(*textline_p)[(*gt_zone_count_p)][0].v_merge = NO;
      			for(p = 0; p < MAX_SG; p++)
				(*textline_p)[(*gt_zone_count_p)][0].SG_ID[p] = -1;
      			(*textline_count_p)[(*gt_zone_count_p)] = 1;
    		}
    		else {
      			(*textline_p)[(*gt_zone_count_p)] =
				(TEXTLINE *)malloc(c * sizeof(TEXTLINE));
      			(*textline_count_p)[(*gt_zone_count_p)] = c;
    		}
    		child = i_GetChildEntity(root);
    		i = 0;
   		while(i < c) {
      			for(p = 0; p < 100; p++)
				(*textline_p)[(*gt_zone_count_p)][i].SG_ID[p]=-1;
      			i_GetArea(child, &data, &type);
      			switch(type) {
      				case iABox:
					(*textline_p)[(*gt_zone_count_p)][i].box =
							*(iBox *)data;
					(*textline_p)[(*gt_zone_count_p)][i].sg_index = 0;
					break;
      				default:
				break;
      			}
      			(*textline_p)[(*gt_zone_count_p)][i].miss_det = NO;
      			(*textline_p)[(*gt_zone_count_p)][i].h_split = NO;
      			(*textline_p)[(*gt_zone_count_p)][i].h_merge = NO;
      			(*textline_p)[(*gt_zone_count_p)][i].v_split = NO;
      			(*textline_p)[(*gt_zone_count_p)][i].v_merge = NO;
      			(*textline_p)[(*gt_zone_count_p)][i].textline_flg = YES;
      			i++;
      			child = i_GetNextEntity(child);
    		}
    		(*gt_zone_count_p)++;
  	}
  	for(child = i_GetChildEntity(root); child; child = i_GetNextEntity(child))
    		GetTextlineInfo(child, gt_zone_count_p, gt_zone_p,
			textline_count_p, textline_p);
 	return(0);
}

/* > */


/* < Load Segmented zone information */
int GetSGZoneInfo(iEntityPtr root, int *sg_zone_count_p, ZONE **sg_zone_p)
{
  	const char *ch = "Paragraph";
  	iEntityPtr child;
  	void *data;
  	iATag type;

  	if(strcmp(ch,i_TypeToName(i_GetEntityType(root))) == 0) {
    		i_GetArea(root, &data, &type);
    		switch(type) {
    			case iABox:
      				(*sg_zone_p)[(*sg_zone_count_p)].box = *(iBox *)data;
    				break;
    			default:
				break;
    		}
    		(*sg_zone_p)[(*sg_zone_count_p)].false_alarm = YES;
    		(*sg_zone_p)[(*sg_zone_count_p)].line_counter = i_CountChildren(root);
    		(*sg_zone_count_p)++;
  	}
  	for(child = i_GetChildEntity(root); child; child = i_GetNextEntity(child))
    		GetSGZoneInfo(child, sg_zone_count_p, sg_zone_p);
  	return(0);
}

/* > */


/* <  Compare ground-truth with Segmentation Results */
void CmpGt_Sg(BENCH_PARAM *para, char *gtdafs, TEXTLINE **textline,
	   iBox *gt_zone, ZONE *sg_zone, int gt_zone_count,
	   int sg_zone_count, int *textline_count, ERR_MEASURE **err_measure_p )
{
  	int num_hsplits = 0;
  	int num_hmerges = 0;
  	int num_miss_det = 0;
  	int num_false_alarm = 0;

  	int textline_hsplit = 0;
  	int textline_hmerge = 0;
  	int textline_miss_det = 0;
  	int textline_false_alarm = 0;

  	int i = 0, j = 0;
  	int num_gt_textlines = 0;
  	int error_gt_textlines = 0;

  	num_hsplits = DetectSplit(para, &textline, &sg_zone, gt_zone_count,
				sg_zone_count, textline_count);
  	num_hmerges = DetectMerge(para, &textline, gt_zone, &sg_zone,
			gt_zone_count, sg_zone_count, textline_count);
  	num_miss_det = DetectMissDet(para, &textline, textline_count, gt_zone_count);
  	num_false_alarm = DetectFalseAlarm(para, sg_zone, sg_zone_count);

  	for(i = 0; i < gt_zone_count; i++) {
    		if(textline_count[i] != 1)
      			num_gt_textlines += textline_count[i];
    		else if (textline_count[i] == 1)
      			if (textline[i][0].textline_flg == YES)
		num_gt_textlines++;
  	}
  	for(i = 0; i < gt_zone_count; i++) {
    		for(j = 0; j < textline_count[i]; j++) {
      			if(textline[i][j].miss_det == YES ||
	 			textline[i][j].h_split == YES ||
	 			textline[i][j].h_merge == YES ) {
				error_gt_textlines++;
				if( textline[i][j].miss_det == YES )
	  				textline_miss_det++;
				if ( textline[i][j].h_split == YES )
	  				textline_hsplit++;
				if (textline[i][j].h_merge == YES)
	  				textline_hmerge++;
      			}
    		}
  	}
  	for(i=0;i<sg_zone_count;i++) {
    		if( sg_zone[i].false_alarm == YES ) {
      			textline_false_alarm+=sg_zone[i].line_counter;
    		}
  	}
   	(*err_measure_p)->nSpl 		= num_hsplits;
   	(*err_measure_p)->nMrg 		= num_hmerges;
   	(*err_measure_p)->nMis		= num_miss_det;
   	(*err_measure_p)->nFA 		= num_false_alarm;
   	(*err_measure_p)->nSplLine    	= textline_hsplit;        
   	(*err_measure_p)->nMrgLine   	= textline_hmerge;
   	(*err_measure_p)->nMisLine	= textline_miss_det;
   	(*err_measure_p)->nFAZone	= textline_false_alarm;
   	(*err_measure_p)->nErrLine	= error_gt_textlines;
   	(*err_measure_p)->nGtLine	= num_gt_textlines;

}

/* > */


/* < Detecting Splitted Textlines */
int DetectSplit(BENCH_PARAM *para, TEXTLINE ***textline_p, ZONE **sg_zone_p,
		int gt_zone_count, int sg_zone_count, int *textline_count)
{
  	int i=0, j=0, k=0;
  	int num_hsplits=0;
  	int split_index=0;
  	int enclose_index=0;

  	for(i = 0; i < gt_zone_count; i++) {
    		for(j = 0; j < textline_count[i]; j++) {
      			for(k = 0; k < sg_zone_count; k++) {
				if(UnionOf(para, (*sg_zone_p)[k],(*textline_p)[i][j])
					== ENCLOSE) {
	  				if((*textline_p)[i][j].textline_flg == YES) {
	    					(*textline_p)[i][j].SG_ID[(*textline_p)[i][j].sg_index] = k;
	    (*textline_p)[i][j].SG_TYPE[(*textline_p)[i][j].sg_index] = ENCLOSE;
	    (*textline_p)[i][j].sg_index++;
	  }
	  (*sg_zone_p)[k].false_alarm=NO;
	}
	else if( UnionOf(para, (*sg_zone_p)[k],(*textline_p)[i][j]) == SPLIT) {
	  if((*textline_p)[i][j].textline_flg == YES) {
	    (*textline_p)[i][j].SG_ID[(*textline_p)[i][j].sg_index] = k;
	    (*textline_p)[i][j].SG_TYPE[(*textline_p)[i][j].sg_index] = SPLIT;
	    (*textline_p)[i][j].sg_index++;
	    num_hsplits++;
	    (*textline_p)[i][j].h_split=YES;
	  }
	  (*sg_zone_p)[k].false_alarm=NO;
	}
      }
    }
  }

  for(i = 0; i < gt_zone_count; i++) {
    for(j = 0; j < textline_count[i]; j++) {
      for(k = 0; k < (*textline_p)[i][j].sg_index; k++) {
	if((*textline_p)[i][j].SG_TYPE[k] == SPLIT)
	  split_index++;
	else if((*textline_p)[i][j].SG_TYPE[k] == ENCLOSE)
	  enclose_index++;
      }
      if(split_index >= 2 && enclose_index ==0 )
	num_hsplits--;
      split_index=0;
      enclose_index=0;
    }
  }

return (num_hsplits);

}

/* > */


/* < Detecting Merged Textlines */
int DetectMerge( BENCH_PARAM *para, TEXTLINE ***textline_p, iBox *gt_zone,
		ZONE **sg_zone_p, int gt_zone_count, int sg_zone_count,
		int *textline_count)
{
  int i, j, p, q, v, w;
  int l1, l2, h1, h2;
  int o_x, o_y;
  int num_hmerges=0;

  for(i = 0; i < gt_zone_count; i++) {
    for(j = 0; j < textline_count[i]; j++) {
      for(p = 0; p < gt_zone_count; p++) {
	for(q = 0; q < textline_count[p]; q++) {
          if( !(p == i && q == j)) {
	    for(v = 0; v < (*textline_p)[i][j].sg_index; v++) {
	      for(w = 0; w < (*textline_p)[p][q].sg_index; w++) {
	        if( (*textline_p)[i][j].SG_ID[v] == 
		  (*textline_p)[p][q].SG_ID[w] && (i != p) ) {
                  l1 = gt_zone[i].w + gt_zone[p].w;
                  l2 = max(gt_zone[i].x + gt_zone[i].w, gt_zone[p].x + gt_zone[p].w) - min(gt_zone[i].x, gt_zone[p].x);
                  h1 = gt_zone[i].h + gt_zone[p].h;
                  h2 = max(gt_zone[i].y + gt_zone[i].h, gt_zone[p].y + gt_zone[p].h) - min(gt_zone[i].y, gt_zone[p].y);
                  o_x = l1 - l2;
                  o_y = h1 - h2;
                
		  if( (o_y*100/gt_zone[i].h <= 100 - para->vtol && o_y <= para->vpix) &&
		     (o_y*100/gt_zone[p].h <= 100 - para->vtol && o_y <= para->vpix) )
		    { } 

                  /* Song's added code */
                  else if ( (*textline_p)[i][j].box.y >= (gt_zone[p].y + gt_zone[p].h - para->vpix) ||
                            (*textline_p)[p][q].box.y >= (gt_zone[i].y + gt_zone[i].h - para->vpix) ||
                            ((*textline_p)[i][j].box.y + (*textline_p)[i][j].box.h) <= (gt_zone[p].y + para->vpix) ||
                            ((*textline_p)[p][q].box.y + (*textline_p)[p][q].box.h) <= (gt_zone[i].y + para->vpix) )
                    { }
                  /* End of Song's added code */

                  else {
                    if((*textline_p)[i][j].h_merge == YES && (*textline_p)[p][q].h_merge != YES) {
                      (*textline_p)[p][q].h_merge = YES;
                      num_hmerges++; 
                    }
                    else if((*textline_p)[i][j].h_merge != YES && (*textline_p)[p][q].h_merge == YES) {
                         (*textline_p)[i][j].h_merge=YES;
                      num_hmerges++;
                    }
                    else if((*textline_p)[i][j].h_merge != YES && (*textline_p)[p][q].h_merge != YES) {
                      (*textline_p)[i][j].h_merge=YES;
                      (*textline_p)[p][q].h_merge=YES;
                      num_hmerges++;
                    }
                    (*textline_p)[i][j].SG_TYPE[v]=MERGE;
                    (*textline_p)[p][q].SG_TYPE[w]=MERGE;
                  }
                }
	      }
	    }
          }
	}
      }
    }
  }

return(num_hmerges);
}
/* > */


/* < Detecting Miss-Detected Textlines */
int DetectMissDet(BENCH_PARAM *para, TEXTLINE ***textline_p, int *textline_count, int gt_zone_count)
{
  int i, j;
  int num_miss_det=0;

  for(i=0;i<gt_zone_count;i++) {
    for(j=0;j<textline_count[i];j++) {
      if((*textline_p)[i][j].sg_index == 0 && (*textline_p)[i][j].textline_flg== YES) {
	(*textline_p)[i][j].miss_det=YES;
	num_miss_det++;
      }
    }
  }
return(num_miss_det);
}

/* > */


/* < Detecting False Alarm Zones */
int DetectFalseAlarm(BENCH_PARAM *para, ZONE *sg_zone, int sg_zone_count)
{
  int i=0;
  int num_false_alarm=0;

  for(i=0;i<sg_zone_count;i++) {
    if( sg_zone[i].false_alarm == YES )
      num_false_alarm++;                    /*=sg_zone[i].line_counter;*/
  }

  return(num_false_alarm);

}
/* > */


/* < Calculating the union between segmentation zone and ground-truth textlines */
int UnionOf(BENCH_PARAM *para, ZONE a, TEXTLINE b)
{
  int o_x, o_y;
  int a_x, a_y;
  int b_x, b_y;
  int l1, l2;
  int h1, h2;

  l1 = a.box.w + b.box.w;
  l2 = max(a.box.x + a.box.w, b.box.x + b.box.w) - min(a.box.x, b.box.x);
  h1 = a.box.h + b.box.h;
  h2 = max(a.box.y + a.box.h, b.box.y + b.box.h) - min(a.box.y, b.box.y);

  o_x = l1 - l2;
  o_y = h1 - h2;

  if(o_x <= 0 || o_y <= 0)
    return(APART);
  else {
    if( o_x == min(a.box.w, b.box.w) && o_x == b.box.w && o_y == min(a.box.h, b.box.h) && o_y == b.box.h )
      return(ENCLOSE);
    else {   
      b_x = o_x * 100 / b.box.w;
      b_y = o_y * 100 / b.box.h;
      a_x = o_x * 100 / a.box.w;
      a_y = o_y * 100 / a.box.h;

      if( b_x > para->htol && b_y > para->vtol && (b.box.w - o_x) < para->hpix && (b.box.h - o_y) < para->vpix ) 
        return (ENCLOSE);
      else if( (b_x < (100 - para->htol) || b_y < (100 - para->vtol)) && (o_x < para->hpix || o_y < para->vpix ) && (a_x < (100 - para->htol) || a_y < (100 - para->vtol)))
        return (APART);
      else
        return (SPLIT);
    }
  }
}

/* > */


/* < Generate Error  output */
void GetError(char *img, TEXTLINE **textline, iBox *gt_zone, ZONE *sg_zone, int gt_zone_count, int sg_zone_count, int *textline_count )
{
  int i=0;
  
  /* Generate error lines in different color in XML format { */
  int j=0;
  char *xmlfile;
  FILE *fpxml;

  xmlfile = (char *) malloc (256 * sizeof(char));
  sprintf(xmlfile, "%s.xml", img);
  fpxml=fopen(xmlfile,"w"); 

  fprintf(fpxml, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
  fprintf(fpxml, "<!DOCTYPE Page SYSTEM \"Toolkit.dtd\">\n");
  fprintf(fpxml, "<Page>\n");
  fprintf(fpxml, "  <PageID Value=\"%d\"> </PageID>\n", 1);
  fprintf(fpxml, "  <PageType Value=\"Journal\"> </PageType>\n");
  fprintf(fpxml, "  <PageNumber Value=\"%d\"> </PageNumber>\n", 1);
  fprintf(fpxml, "  <PageColumns Value=\"%d\"> </PageColumns>\n", 2);
  fprintf(fpxml, "  <PageNext Value=\"unknown\"> </PageNext>\n");
  fprintf(fpxml, "  <PageZones Value=\"unknown\"> </PageZones>\n");
  fprintf(fpxml, "  <Language Codeset=\"UTF-8\" Script=\"Roman\" Type=\"English\"> </Language>\n");
  fprintf(fpxml, "  <Font Size=\"9-12\" Spacing=\"Undefined\" Style=\"Normal\" Type=\"Serif\"> </Font>\n");
  fprintf(fpxml, "  <Alignment Value=\"Left\"> </Alignment>\n");
  fprintf(fpxml, "  <ReadingDir Value=\"LeftRight\"> </ReadingDir>\n");
  fprintf(fpxml, "  <CharOrient Value=\"UpRight\"> </CharOrient>\n");
  fprintf(fpxml, "  <GT_Text Value=\"\"> </GT_Text>\n");
  fprintf(fpxml, "  <Classification>\n");
  fprintf(fpxml, "  <Category Value=\"\"> </Category>\n");
  fprintf(fpxml, "  <Type Value=\"\"> </Type>\n");
  fprintf(fpxml, "  </Classification>\n");

  for(i=0;i<gt_zone_count;i++) { 
    for(j=0;j<textline_count[i];j++) { 
	if( textline[i][j].miss_det == YES && textline[i][j].textline_flg == YES) {
		fprintf(fpxml, "<Zone>\n");
  		fprintf(fpxml, "<ZoneID Value=\"%d\"/>\n", j);
  		fprintf(fpxml, "  <Classification>\n");
  		fprintf(fpxml, "    <Category Value=\"mis-detection\"> </Category>\n");
  		fprintf(fpxml, "    <Type Value=\"Text\"> </Type>\n");
  		fprintf(fpxml, "  </Classification>\n");
  		fprintf(fpxml, "  <Color Value=\"red\"></Color>\n");
  		fprintf(fpxml, "  <ZoneCorners>\n");
  		fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
				textline[i][j].box.x, textline[i][j].box.y);
  		fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                  		textline[i][j].box.x + textline[i][j].box.w, textline[i][j].box.y);
  		fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                  		textline[i][j].box.x + textline[i][j].box.w,
				textline[i][j].box.y + textline[i][j].box.h);
  		fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                  		textline[i][j].box.x, textline[i][j].box.y + textline[i][j].box.h);
  		fprintf(fpxml, "  </ZoneCorners>\n");
  		fprintf(fpxml, "  <GT_Text Value=\"a\"></GT_Text>\n");
		fprintf(fpxml, "</Zone>\n");
	}
        else if( (textline[i][j].h_split == YES) && 
	  	 (textline[i][j].h_merge == NO) && 
	  	 (textline[i][j].textline_flg == YES) ) { 
		fprintf(fpxml, "<Zone>\n");
                fprintf(fpxml, "<ZoneID Value=\"%d\"/>\n", j);
                fprintf(fpxml, "  <Classification>\n");
                fprintf(fpxml, "    <Category Value=\"split\"> </Category>\n");
                fprintf(fpxml, "    <Type Value=\"Text\"> </Type>\n");
                fprintf(fpxml, "  </Classification>\n");
                fprintf(fpxml, "  <Color Value=\"blue\"></Color>\n");
                fprintf(fpxml, "  <ZoneCorners>\n");
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x, textline[i][j].box.y);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x + textline[i][j].box.w, textline[i][j].box.y);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x + textline[i][j].box.w,
                                textline[i][j].box.y + textline[i][j].box.h);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x, textline[i][j].box.y + textline[i][j].box.h);
                fprintf(fpxml, "  </ZoneCorners>\n");
                fprintf(fpxml, "  <GT_Text Value=\"a\"></GT_Text>\n");
                fprintf(fpxml, "</Zone>\n");

      	}
	else if((textline[i][j].h_split == NO) &&
                 (textline[i][j].h_merge == YES) &&
                 (textline[i][j].textline_flg == YES) ) { 
		fprintf(fpxml, "<Zone>\n");
                fprintf(fpxml, "<ZoneID Value=\"%d\"/>\n", j);
                fprintf(fpxml, "  <Classification>\n");
                fprintf(fpxml, "    <Category Value=\"horizontally-merge\"> </Category>\n");
                fprintf(fpxml, "    <Type Value=\"Text\"> </Type>\n");
                fprintf(fpxml, "  </Classification>\n");
                fprintf(fpxml, "  <Color Value=\"orange\"></Color>\n");
                fprintf(fpxml, "  <ZoneCorners>\n");
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x, textline[i][j].box.y);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x + textline[i][j].box.w, textline[i][j].box.y);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x + textline[i][j].box.w,
                                textline[i][j].box.y + textline[i][j].box.h);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x, textline[i][j].box.y + textline[i][j].box.h);
                fprintf(fpxml, "  </ZoneCorners>\n");
                fprintf(fpxml, "  <GT_Text Value=\"a\"></GT_Text>\n");
                fprintf(fpxml, "</Zone>\n");

	} 
	else if((textline[i][j].h_split == YES) &&
                 (textline[i][j].h_merge == YES) &&
                 (textline[i][j].textline_flg == YES) ) {
                fprintf(fpxml, "<Zone>\n");
                fprintf(fpxml, "<ZoneID Value=\"%d\"/>\n", j);
                fprintf(fpxml, "  <Classification>\n");
                fprintf(fpxml, "    <Category Value=\"split plus horizontally merge\"> </Category>\n");
                fprintf(fpxml, "    <Type Value=\"Text\"> </Type>\n");
                fprintf(fpxml, "  </Classification>\n");
                fprintf(fpxml, "  <Color Value=\"magenta\"></Color>\n");
                fprintf(fpxml, "  <ZoneCorners>\n");
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x, textline[i][j].box.y);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x + textline[i][j].box.w, textline[i][j].box.y);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x + textline[i][j].box.w,
                                textline[i][j].box.y + textline[i][j].box.h);
                fprintf(fpxml, "    <Vertex x=\"%d\" y=\"%d\"></Vertex>\n",
                                textline[i][j].box.x, textline[i][j].box.y + textline[i][j].box.h);
                fprintf(fpxml, "  </ZoneCorners>\n");
                fprintf(fpxml, "  <GT_Text Value=\"a\"></GT_Text>\n");
                fprintf(fpxml, "</Zone>\n");

        } 

    } 
  } 
  fprintf(fpxml, "</Page>\n");
  fclose(fpxml);
  free(xmlfile);

  /*
  for(i=0;i<sg_zone_count;i++) { 
    if(sg_zone[i].false_alarm == YES) 
      fprintf(fp, "%d\t%d\t%d\t%d\n", sg_zone[i].box.x, sg_zone[i].box.y, 
		sg_zone[i].box.w, sg_zone[i].box.h); 
  }
  */

/* Generate error lines in different color in XML format } */

for(i = 0; i < gt_zone_count; i++)
  free(textline[i]);
free(textline);

free(sg_zone);
free(gt_zone);
free(textline_count);


}

/* > */
