/*  
**  File: DocstrumFunction.c
**  Purpose: Docstrum segmentation algorithm function implementation
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  References: The Document Spectrum for Page Layout Analysis ,
**              IEEE PAMI Vol. 15, No. 11, Nov. 1993 pp. 1162-1173
**  $Id: DocstrumFunction.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <math.h>
#include <string.h>

#include <sys/types.h>
#include <sys/uio.h>
#include <sys/ddi.h>
#include "rafport.h"
#include "dafstype.h"
#include "rlimage.h"
#include "dafs.h"

#include <stdlib.h>
#include "tiffio.h"
#include "tiff.h"
#include <time.h>

#ifdef SUNOS
#include "libfun.h"
#endif

#include "pset.h"

int DocstrumFunction(char *tif_file, char *cc_file, DOCSTRUM_PARAM *para, char *dafs_file)
{
  	EST_PARA est_para;
  	RECT_DOCSTRUM *document;
  	RECT_DOCSTRUM *zone;
  	RECT_DOCSTRUM *line;
  	RECT_DOCSTRUM *comp, *comp1;

  	DIS **d;
  	int cc_index = 0, cc_index1 = 0;
  	int cc_height = 0, cc_height1 = 0;
  	int line_index = 0;
  	int zone_index = 0;

  	int line_t = 0;
  	int par_t;
  	int per_t;
  	int par_t_h;
  	int per_t_h;

  	int rerun;
  	int runs = 0;
  	float height_ratio;

  	est_para.or = 0;
  	est_para.cs = 0;
  	est_para.ts = 0;

  	ReadTIFFImageDocstrum(tif_file, &document, &line, &zone, &para);

  	ReadCC(cc_file, para, &cc_index, &cc_index1, &cc_height, &cc_height1,
		&comp, &comp1, &d, &rerun, &height_ratio);

  	EstPara(runs, document, comp, &d, cc_index, cc_height, height_ratio,
		para, &est_para, &line_t, &par_t, &per_t, &par_t_h, &per_t_h);

  	MakeTextline(est_para, para, comp, d, cc_index, line_t, &line_index, &line);
 
  	if (rerun == 1 && cc_index1 > 1) {
    		runs ++;
    		EstPara(runs, document, comp1, &d, cc_index1, cc_height1,
			height_ratio, para, &est_para, &line_t, &par_t,
			&per_t, &par_t_h, &per_t_h);

    		MakeTextline(est_para, para, comp1, d, cc_index1,
				line_t, &line_index, &line);
  	}
  	else if( cc_index1 == 1) {
    		free(comp1);
  	}

  	MakeZone(est_para, para, line, line_index, par_t, per_t, par_t_h,
		per_t_h, &zone_index, &zone); 

  	GetDAFS(tif_file, dafs_file, line_index, zone_index, document, line, zone);

  	FreeJobDocstrum(zone, document);

  	return(0);

} 




/* < Euclidean distance */
float dis(RECT_DOCSTRUM a, RECT_DOCSTRUM b)
{
  	float d;
  	float x0, y0, x1, y1;
  
  	x0 = a.x1 + 0.5 * (a.x2 - a.x1);
  	y0 = a.y1 + 0.5 * (a.y2 - a.y1);
  	x1 = b.x1 + 0.5 * (b.x2 - b.x1);
  	y1 = b.y1 + 0.5 * (b.y2 - b.y1);
  
  	d = sqrt((x0 - x1) * (x0 - x1) + (y0 - y1) * (y0 - y1)); 
  	return(d);
}
/* > */

/* < angle */
float ang(RECT_DOCSTRUM a, RECT_DOCSTRUM b)
{  
  	float al;
  	float x0, y0, x1, y1;
  
  	x0 = a.x1 + 0.5 * (a.x2 - a.x1);
  	y0 = a.y1 + 0.5 * (a.y2 - a.y1);
  	x1 = b.x1 + 0.5 * (b.x2 - b.x1);
  	y1 = b.y1 + 0.5 * (b.y2 - b.y1);        
  
  	if(x1 == x0) {
    		if(y0 > y1)
      			al = M_PI_2;
    		else
      			al = -M_PI_2;
  	}
  	else {
    		al = atan((y1 - y0) / (x1 - x0));
  	}
  	return(al);
}

/* > */

/* < parallel distance */
int par_dist(RECT_DOCSTRUM a, RECT_DOCSTRUM b)
{
  	int l1, l2;
  
  	l1 = (a.x2 - a.x1) + (b.x2 - b.x1);
  	l2 = max(a.x2, b.x2) - min(a.x1, b.x1);

  	if(l1 >= l2) 
    		return(0);
  	else
    	return(l2 - l1);
}

/* > */

/* < perpendicular distance */
int per_dist(RECT_DOCSTRUM a, RECT_DOCSTRUM b)
{
  	int l1, l2;
  
  	l1 = (a.y2 - a.y1) + (b.y2 - b.y1);
  	l2 = max(a.y2, b.y2) - min(a.y1, b.y1);

  	if(l1 >= l2) 
    		return(0);
  	else
    		return(l2 - l1);
}

/* > */


/* < READ TIFF IMAGE */
int ReadTIFFImageDocstrum(char *tif_file, RECT_DOCSTRUM **document_p,
		RECT_DOCSTRUM **line_p, RECT_DOCSTRUM **zone_p,
		DOCSTRUM_PARAM **para_p)
{
  	TIFF *tif;
  	int i;
	float floattag;
	int xres, yres;

  	(*document_p) = (RECT_DOCSTRUM *) malloc (sizeof(RECT_DOCSTRUM));
  	(*document_p)->x1 = 0;
  	(*document_p)->y1 = 0;

  	tif = TIFFOpen(tif_file, "r");
  	if (tif == NULL) return(-1);
  	TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &((*document_p)->x2));
  	TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &((*document_p)->y2));

        floattag = 0;
        TIFFGetField(tif, TIFFTAG_XRESOLUTION, &floattag);
        xres = (int) floattag;

        floattag = 0;
        TIFFGetField(tif, TIFFTAG_YRESOLUTION, &floattag);
        yres = (int) floattag;

        (*para_p)->lsize = (*para_p)->lsize * yres / 300;
        (*para_p)->hsize = (*para_p)->hsize * yres / 300;

  	TIFFClose(tif);

  	*line_p = (RECT_DOCSTRUM *) malloc (MAX_RECTS * sizeof(RECT_DOCSTRUM));
  	*zone_p = (RECT_DOCSTRUM *) malloc (MAX_RECTS * sizeof(RECT_DOCSTRUM));
  	for(i = 0; i < MAX_RECTS; i++) {
    		(*line_p)[i].x1 = (*line_p)[i].y1 = 10000;
    		(*line_p)[i].x2 = (*line_p)[i].y2 = 0;
    		(*zone_p)[i].x1 = (*zone_p)[i].y1 = 10000;
    		(*zone_p)[i].x2 = (*zone_p)[i].y2 = 0;
  	}

  	return(0);
}

/* > */
 
/* < Read Connected Components */
void ReadCC(char *cc_file, DOCSTRUM_PARAM *para, int *cc_index_p, int *cc_index1_p,
		int *cc_height_p, int *cc_height1_p, RECT_DOCSTRUM **comp_p,
		RECT_DOCSTRUM **comp1_p, DIS ***d_p, int *rerun_p,
		float *height_ratio_p)
{
  	int c0, c1, c2, c3;
  	int avg_height = 0;
  	FILE *fp;
  	char c;
  	char *tmp;
  	tmp = (char *) malloc (256 * sizeof(char));

  	fp = fopen(cc_file,"r");
  	while( (c = fgetc(fp)) == '#' ) {
    		fgets(tmp, 256, fp);
  	}

  	ungetc(c, fp);

  	while ( !feof(fp) ) {
    		fscanf(fp,"%d%d%d%d\n", &c0, &c1, &c2, &c3);
    		if((c2-c0) > para->lsize && (c3-c1) > para->lsize &&
			(c2-c0) < para->hsize && (c3-c1) < para->hsize) {
      			(*cc_index_p)++;
      			avg_height += c3 - c1;
    		}
  	}
  	avg_height = avg_height / (*cc_index_p);
  	rewind(fp);

  	(*comp_p) = (RECT_DOCSTRUM *) malloc ((*cc_index_p) * sizeof(RECT_DOCSTRUM));
  	(*comp1_p) = (RECT_DOCSTRUM *) malloc ((*cc_index_p) * sizeof(RECT_DOCSTRUM));
  
 	(*cc_index_p) = 0;
  	(*cc_index1_p) = 0;
  	(*cc_height_p) = 0;
  	(*cc_height1_p) = 0;

  	while( (c = fgetc(fp)) == '#' ) {
    		fgets(tmp, 256, fp);
  	}

  	free(tmp);
  	ungetc(c, fp);

  	while ( !feof(fp) ) {
    		fscanf(fp,"%d%d%d%d\n", &c0, &c1, &c2, &c3);
    		if((c2-c0)>para->lsize && (c3-c1)>para->lsize &&
			(c2-c0)<para->hsize && (c3-c1)<para->hsize) {
      			if( (c3 - c1) < (para->dis * avg_height) ) {
        			(*comp_p)[(*cc_index_p)].x1 = c0;
        			(*comp_p)[(*cc_index_p)].y1 = c1;
        			(*comp_p)[(*cc_index_p)].x2 = c2;
        			(*comp_p)[(*cc_index_p)].y2 = c3;
        			(*comp_p)[(*cc_index_p)].flg = 0;  
        			(*cc_height_p) = (*cc_height_p) + c3 - c1;
        			(*cc_index_p)++;
      			}
      			else {
        			(*comp1_p)[(*cc_index1_p)].x1 = c0;
        			(*comp1_p)[(*cc_index1_p)].y1 = c1;
        			(*comp1_p)[(*cc_index1_p)].x2 = c2;
        			(*comp1_p)[(*cc_index1_p)].y2 = c3;
        			(*comp1_p)[(*cc_index1_p)].flg = 0;  
        			(*cc_height1_p) = (*cc_height1_p) + c3 - c1;
        			(*cc_index1_p)++;      
      			}
    		}
  	}

  	fclose(fp);

  	if( *cc_index1_p == 0 ) { 
    		free((*comp1_p));
    		(*rerun_p) = 0;
    		(*cc_height_p) = (*cc_height_p) / (*cc_index_p);
    		(*height_ratio_p) = 1;
  	}
  	else {
    		(*rerun_p) = 1;
    		(*cc_height_p) = (*cc_height_p) / (*cc_index_p);
    		(*cc_height1_p) = (*cc_height1_p) / (*cc_index1_p);
    		(*height_ratio_p) = (*cc_height1_p) / (*cc_height_p);
  	}
}

/* > */

/* < Estimate Orientation, Character Spacing and Textline Spacing */ 
void EstPara(int runs, RECT_DOCSTRUM *document, RECT_DOCSTRUM *comp,
		DIS ***d_p, int cc_index, int cc_height, float height_ratio,
		DOCSTRUM_PARAM *para, EST_PARA *est_para_p, int *line_t_p,
		int *par_t_p, int *per_t_p, int *par_t_h_p, int *per_t_h_p)
{
  	int    i, j, m;
  	int    *a_h,*w_h, *b_h;
  	int    a_index , max_a = 0;
  	int    d_tmp = 0;
  	int    max_w = 0, max_b = 0;
  	int    tmp = 0;
  	float  l_h_a, s_h_a;
  	float  l_v_a, s_v_a;

  	float *comp_x;
  	int   *comp_i;
  	float *d_array;
  	int   *i_array;
  	int   s_pos, e_pos;
  	int   pre_s_pos = 0, pre_e_pos = 0;
  	int   flg = 0;
  	int   n;
  	int   x_dis;
  	float l_x, r_x;

  	int   k;

  	tmp = (int) (sqrt((double)(document->x2 * document->x2 + document->y2 * document->y2)) + 1);
  	a_h = (int *) malloc (181 * sizeof(int));
  	(*d_p) = (DIS **) malloc (cc_index * sizeof(DIS *));
  	for(i = 0; i < cc_index; i++)
    		(*d_p)[i] = (DIS *) malloc (para->knn * sizeof(DIS));
  	for (i = 0; i < 181; i++) 
    		a_h[i] = 0;
  	/* sort connected component by there x coordinates */
  	comp_x = (float *) malloc (cc_index * sizeof(float));
  	comp_i = (int *) malloc (cc_index * sizeof(int));

  	for(i = 0; i < cc_index; i++) {
    		comp_x[i] = comp[i].x1 + 0.5 * (comp[i].x2 - comp[i].x1);
    		comp_i[i] = i;
 	}

  	sort2(cc_index, --comp_x, --comp_i);
  	comp_x++;
  	comp_i++;

  	d_array = (float *) malloc (cc_index * sizeof(float));
  	i_array = (int *)   malloc (cc_index * sizeof(int));

  	if(cc_index <= para->knn)
    		k = cc_index - 1;
  	else 
    		k = para->knn;

  	for(i = 0; i < cc_index; i++) {
    		x_dis = 1;
    		while(1) {
      			l_x = comp_x[i] - x_dis;
      			r_x = comp_x[i] + x_dis;
      			if(l_x <= comp_x[0])
        			s_pos = 0;
      			else {
        			m = 0;
       				while(!(comp_x[i - m - 1] < l_x && comp_x[i - m] >= l_x))
          			m++;
        			s_pos = i - m;
      			}
      			if(r_x >= comp_x[cc_index - 1])
        			e_pos = cc_index - 1;
      			else {
        			m = 0;
        			while(!(comp_x[i + m + 1] > r_x && comp_x[i + m] <= r_x))
         				m++;
        			e_pos = i + m;
      			}
      			if(e_pos - s_pos >= k) {
        			if(flg == 0) {
          				n = 0;
          				for(m = s_pos; m <= e_pos; m++) {
            					if(m != i) {
              						if(dis(comp[comp_i[i]],
								comp[comp_i[m]]) == -1) {   
                						d_array[n] =
									dis(comp[comp_i[i]],
									comp[comp_i[m]]); 
              						}
              						else
                						d_array[n] =
									dis(comp[comp_i[i]],
									comp[comp_i[m]]); 
              						i_array[n] = comp_i[m];
              						n++;
            					}
          				}
          				selectK(--d_array, --i_array, n, k);
          				d_array++;
          				i_array++;
          				pre_s_pos = s_pos;
          				pre_e_pos = e_pos;
          				flg = 1;
          				if(d_array[k - 1] > x_dis) {
            					x_dis++;
          				}
          				else {
            					flg = 0;
            					break;
          				}
        			}
        			else if(flg == 1) {
          				n = k;
          				if(pre_s_pos != 0) {
            					for(m = s_pos; m < pre_s_pos; m++) {
              						if(dis(comp[comp_i[i]],
								comp[comp_i[m]]) == -1) {   
                						d_array[n] =
									dis(comp[comp_i[i]],
									comp[comp_i[m]]); 
              						}
              						else
                						d_array[n] =
									dis(comp[comp_i[i]],
									comp[comp_i[m]]);  
              						i_array[n] = comp_i[m];
              						n++;
            					}
          				}
          				if(pre_e_pos < cc_index - 1) {
            					for(m = pre_e_pos + 1; m <= e_pos; m++) {
              						if(dis(comp[comp_i[i]],
								comp[comp_i[m]]) == -1) {   
                						d_array[n] =
									dis(comp[comp_i[i]],
									comp[comp_i[m]]);  
              						}
              						else
                						d_array[n] =
									dis(comp[comp_i[i]],
									comp[comp_i[m]]);  
              						i_array[n] = comp_i[m];
              						n++;
            					}
          				}
          				selectK(--d_array, --i_array, n, k);
          				d_array++;
          				i_array++;
          				pre_s_pos = s_pos;
          				pre_e_pos = e_pos;
 
 	         			if(d_array[k - 1] > x_dis) {
  	          				x_dis++;
   		      			}
         	 			else {
          	  				flg = 0;
           	 				break;
          				}
        			}
      			}
      			else
        			x_dis++;
    		}

    		for(j = 0; j < k; j++) {
      			(*d_p)[comp_i[i]][j].d = d_array[j];
      			(*d_p)[comp_i[i]][j].a = ang(comp[comp_i[i]],comp[i_array[j]]);
      			(*d_p)[comp_i[i]][j].i = i_array[j];

      			a_index = (int) (((*d_p)[comp_i[i]][j].a) * 180/M_PI);
      			a_h[a_index + 90]++;
      			if(a_h[a_index + 90] > max_a) {
        			max_a = a_h[a_index + 90];
        			(*est_para_p).or = a_index;
      			}
    		}
  	}

  	l_h_a = ((*est_para_p).or + para->hatol) * M_PI / 180;
  	s_h_a = ((*est_para_p).or - para->hatol) * M_PI / 180;
  	l_v_a = ((*est_para_p).or + 90 + para->vatol) * M_PI / 180;
  	s_v_a = ((*est_para_p).or + 90 - para->vatol) * M_PI / 180;

  	free(d_array);
  	free(i_array);
  	free(comp_x);
  	free(comp_i);

  	if( runs == 0 ) { 
    		w_h = (int *) malloc (tmp * sizeof(int));
    		b_h = (int *) malloc (tmp * sizeof(int));
    		for (i = 0; i < tmp; i++) {
      			w_h[i] = 0;
      			b_h[i] = 0;
    		}
    		/**----------make within-line\between-line histogram-----------------**/
    		for(i = 0; i < cc_index; i++) {
      			for(m = 0; m < para->knn; m++)  {
        			d_tmp = (int)((*d_p)[i][m].d);
        			if ((*d_p)[i][m].a < l_h_a &&
  	    				(*d_p)[i][m].a > s_h_a ) {
          				w_h[d_tmp]++;
          				if(w_h[d_tmp] > max_w) {
	    					max_w = w_h[d_tmp];
	    					(*est_para_p).cs = d_tmp;
          				}
        			}      
        			else {
          				if ((*est_para_p).or < 0) {
	    					if(l_v_a > 1.57) {
	      						if( ((*d_p)[i][m].a >= s_v_a &&
							(*d_p)[i][m].a <= M_PI_2 ) ||
	          					((*d_p)[i][m].a >= -M_PI_2 &&
							(*d_p)[i][m].a <= l_v_a - M_PI) ) {
	        						b_h[d_tmp]++;
	        						if(b_h[d_tmp] > max_b) {
	          							max_b = b_h[d_tmp];
	          							(*est_para_p).ts =
										d_tmp;
	        						}
	      						}
	    					}
	    					else {
	      						if( ((*d_p)[i][m].a >= s_v_a) && 
	          						((*d_p)[i][m].a <= l_v_a) ) {
	        						b_h[d_tmp]++;
	        						if(b_h[d_tmp] > max_b) {
	          							max_b = b_h[d_tmp];
	          							(*est_para_p).ts = d_tmp;
	        						}
	      						}
	    					}
          				}
          				else if ((*est_para_p).or > 0) {
	    					if(s_v_a < -1.57) {
	      						if( ((*d_p)[i][m].a <= l_v_a &&
							(*d_p)[i][m].a >= -M_PI_2 ) ||
	          					((*d_p)[i][m].a <= M_PI_2 &&
							(*d_p)[i][m].a >= s_v_a ) ) {
	        					b_h[d_tmp]++;
	        					if(b_h[d_tmp] > max_b) {
	          						max_b = b_h[d_tmp];
	          						(*est_para_p).ts = d_tmp;
	        					}
	      					}
	    				}
	    				else {
	      					if( ((*d_p)[i][m].a >= s_v_a) && 
	          					((*d_p)[i][m].a <= l_v_a) ) {
	        					b_h[d_tmp]++;
	        					if(b_h[d_tmp] > max_b) {
	          						max_b = b_h[d_tmp];
	          						(*est_para_p).ts = d_tmp;
	        					}
	      					}
	    				}
          			}
          			else if ((*est_para_p).or == 0) {
	    				if( ((*d_p)[i][m].a >= -1.57 &&
						(*d_p)[i][m].a <= (-90 + para->vatol) * M_PI / 180 ) ||
	        				((*d_p)[i][m].a >= (90 - para->vatol) * M_PI / 180 &&
						(*d_p)[i][m].a <= 1.57) ) {
	      					b_h[d_tmp]++;
	      					if(b_h[d_tmp] > max_b) {
	        					max_b = b_h[d_tmp];
	        					(*est_para_p).ts = d_tmp;	
	      					}
	    				}
         			}
        			}
      			}
    		}
 
	
    		(*line_t_p) = (int)((*est_para_p).cs * para->tcc); 
    		(*par_t_p) =  (int)((*est_para_p).cs * para->par);  
    		(*per_t_p) =  (int)((*est_para_p).ts * para->per); 

    		free (w_h);
    		free (b_h);
    		free (a_h);
  	}

  	else {
    		(*line_t_p) = (int)((*line_t_p) * height_ratio);
    		(*par_t_h_p) = (int)((*par_t_p) * height_ratio);
    		(*per_t_h_p) = (int)((*per_t_p) * height_ratio);
    		free (a_h);
  	}
}

/* > */

/* < Group connected components into text lines */
void MakeTextline(EST_PARA est_para, DOCSTRUM_PARAM *para, RECT_DOCSTRUM *comp,
		DIS **d, int cc_index, int line_t, int *line_index_p,
		RECT_DOCSTRUM **line_p) 
{
  	int  i, k;
  	int  *x, *y;
  	int  *pre_x, *pre_y;
  	int  E = 0;
  	int  index_check;

  	int  pk;
  	index_check = cc_index;
  	
	x = (int *) malloc ( index_check * sizeof(int));
  	y = (int *) malloc ( index_check * sizeof(int));

  	if(cc_index <= para->knn)
    	pk = cc_index - 1;
  	else 
    	pk = para->knn;
  	E = 0;
  	for(i = 0; i < cc_index; i++) {
    		for(k = 0; k < pk ; k++) {
      			if( ((int)(d[i][k].d) < line_t &&
           			(int)(d[i][k].a * 180/M_PI) >=
				(est_para.or - para->hatol) &&
           			(int)(d[i][k].a * 180/M_PI) <=
				(est_para.or + para->hatol)) || 
           			(int)(d[i][k].d) < (line_t * para->scr) ) {
        			x[E] = i;
        			y[E] = d[i][k].i;
        			E++;
				if(E >= index_check) {
					pre_x = x;
					pre_y = y;
					index_check += cc_index;
					x = (int *) malloc ( index_check * sizeof(int));
  					y = (int *) malloc ( index_check * sizeof(int));
					x = (int *) memcpy(x, pre_x,
						(index_check - cc_index)*sizeof(int));
					y = (int *) memcpy(y, pre_y,
						(index_check - cc_index)*sizeof(int));
					free(pre_x);
					free(pre_y);
				}
      			}
    		}
  	}

  	for(i = 0; i < cc_index; i++)
    		free(d[i]);
  	free(d);

  	TransitiveClosure(E, x, y, comp, 0, cc_index, line_index_p, line_p); 

  	free(x);
  	free(y);
  	free(comp);
}

/* > */

/* < Group text-lines into zones */
void MakeZone(EST_PARA est_para, DOCSTRUM_PARAM *para, RECT_DOCSTRUM *line,
		int line_index, int par_t, int per_t, int par_t_h, int per_t_h,
		int *zone_index_p, RECT_DOCSTRUM **zone_p) 
{
  	int  i, k;
  	int  *x, *y;
  	int  *pre_x, *pre_y;
  	int  E = 0;
  	int  index_check;
  
 	x = (int *) malloc ( (int)(line_index * para->knn * 0.5) * sizeof(int));
  	y = (int *) malloc ( (int)(line_index * para->knn * 0.5) * sizeof(int));

  	index_check = (int)(line_index * para->knn * 0.5); 
  	E = 0;
  	for(i = 0; i < line_index - 1; i++) {
    		for(k = i + 1; k < line_index; k++) {
      			if( ((line[i].flg || line[k].flg) == 1 &&
				par_dist(line[i],line[k]) < par_t_h &&
           			per_dist(line[i],line[k]) < per_t_h) ||
          			((line[i].flg || line[k].flg) == 0 &&
				par_dist(line[i],line[k]) < par_t &&
           			per_dist(line[i],line[k]) < per_t) ) {
        			x[E] = i;
        			y[E] = k;
        			E++;
				if(E >= index_check) {
                			pre_x = x;
                			pre_y = y;
                			index_check += line_index;
                			x = (int *) malloc ( index_check * sizeof(int));
                			y = (int *) malloc ( index_check * sizeof(int));
                			x = (int *) memcpy(x, pre_x, (index_check -
						line_index) *sizeof(int));
                			y = (int *) memcpy(y, pre_y, (index_check -
						line_index) *sizeof(int));
                			free(pre_x);
                			free(pre_y);
        			}
      			} 
    		}
  	}

  	TransitiveClosure(E, x, y, line, 0, line_index, zone_index_p, zone_p); 

  	free(x);
  	free(y);
  	free(line);
}

/* > */
 
/* < Transitive Closure */
void TransitiveClosure(int E, int *x, int *y, RECT_DOCSTRUM *rect1, int rect1_index_s,
		int rect1_index_e, int *rect2_index_p, RECT_DOCSTRUM **rect2_p) 
{
  	int i, j, k, e;
  	int ch_index = 0;
  	NODE_DOCSTRUM *t, *z;
  	NODE_DOCSTRUM **free_t;
  	NODE_DOCSTRUM **adj;
  	int  *val;
  	int  id = 0;
  	int l = 0;

  	adj = (NODE_DOCSTRUM **) malloc (rect1_index_e * sizeof(NODE_DOCSTRUM *));
  	val = (int *) malloc (rect1_index_e * sizeof(int));
  	z = (NODE_DOCSTRUM *) malloc (sizeof(NODE_DOCSTRUM));
  	z->next = z;

  	free_t = (NODE_DOCSTRUM **) malloc ( 2 * E * sizeof(NODE_DOCSTRUM *)); 

  	for(j = rect1_index_s; j < rect1_index_e; j++) 
    		adj[j] = z;
  	for(e = 0; e < E; e++) {
    		t = (NODE_DOCSTRUM *) malloc (sizeof (NODE_DOCSTRUM));
    		free_t[l++] = t; 
    		t->v = x[e]; 
    		t->next = adj[y[e]];
    		adj[y[e]] = t;
    		t = (NODE_DOCSTRUM *) malloc (sizeof(NODE_DOCSTRUM));
    		free_t[l++] = t; 
    		t->v = y[e]; 
    		t->next = adj[x[e]]; 
    		adj[x[e]] = t;
  	}
  	for(i = rect1_index_s; i < rect1_index_e; i++) 
    		val[i] = 0;
  	for(k = rect1_index_s; k < rect1_index_e; k++)
    		if(val[k] == 0) { 
      			ch_index = 0;
      			visit(k, rect1, z, &adj, &id, &val, rect2_index_p,
				rect2_p, &ch_index);
      			if(rect1_index_s != 0)
        			(*rect2_p)[*rect2_index_p].flg = 1;
      			else
        			(*rect2_p)[*rect2_index_p].flg = 0;
      				(*rect2_index_p) ++;
    		}
  	for(i = 0; i < l; i++)
    	free(free_t[i]);
  	free(free_t);
 	free(z);
  	free(adj);
  	free(val);
}

/* > */
 
/* < Recursive visit vertices */
void visit(int k, RECT_DOCSTRUM *rect1, NODE_DOCSTRUM *z, NODE_DOCSTRUM ***adj_p,
	int *id_p, int **val_p, int *index_p, RECT_DOCSTRUM **rect2_p, int *ch_index_p)
{
   	NODE_DOCSTRUM *t;
   	(*val_p)[k] = ++(*id_p);
   	(*rect2_p)[(*index_p)].x1 = min((*rect2_p)[(*index_p)].x1, rect1[k].x1);
   	(*rect2_p)[(*index_p)].y1 = min((*rect2_p)[(*index_p)].y1, rect1[k].y1);
   	(*rect2_p)[(*index_p)].x2 = max((*rect2_p)[(*index_p)].x2, rect1[k].x2);
   	(*rect2_p)[(*index_p)].y2 = max((*rect2_p)[(*index_p)].y2, rect1[k].y2);

   	for(t = (*adj_p)[k]; t !=z; t = t->next)
     		if((*val_p)[t->v] == 0)
			visit(t->v, rect1, z, adj_p, id_p, val_p, index_p,
				rect2_p, ch_index_p);
}

/* > */

/* < Free jobs */
void FreeJobDocstrum(RECT_DOCSTRUM *zone, RECT_DOCSTRUM *document)
{
  	free(zone);
  	free(document);
}

/* > */


/* < Select K nearest connected components */
void selectK(float a[], int b[], int n, int k)
{
  	float v,t;
  	int   p, i, j, l, r;
  
  	l = 1; r = n;
  	while(r > l) {
    		v = a[r];
		i = l - 1;
		j = r;
    		for(;;) {
      			while(a[++i] < v);
      			while(a[--j] > v && j > 1);
      			if(i >= j)
				break;
      			t = a[i];
			a[i] = a[j];
			a[j] = t;
      			p = b[i];
			b[i] = b[j];
			b[j] = p;
    		}
    		t = a[i];
		a[i] = a[r];
		a[r] = t;
    		p = b[i];
		b[i] = b[r];
		b[r] = p;
    		if(i >= k)
			r = i - 1;
    		if(i <= k)
			l = i + 1;
  	}
}

/* > */ 

/* < Create DAFS */
void GetDAFS(char *tif_file, char *dafs_file, int line_index, int zone_index,
		RECT_DOCSTRUM *document, RECT_DOCSTRUM *line, RECT_DOCSTRUM *zone)
{
  	iEntityPtr dafs_doc;
  	iEntityPtr dafs_zone;
  	iDAFSError dafs_error;
  	iBox dafs_box;
  	iImagePtr imPtr;

  	int i;

  	dafs_error = i_InitLib();
  	dafs_doc = i_NewEntity(iDocumentT, 0);
  	dafs_box.x = 0;
  	dafs_box.y = 0;
  	dafs_box.w = document->x2;
  	dafs_box.h = document->y2;
  	i_SetBox(dafs_doc,dafs_box);

  	for(i = 0; i < zone_index; i++) {
    		dafs_zone = i_NewEntity(iParagraphT, dafs_doc);
    		dafs_box.x = zone[i].x1;
    		dafs_box.y = zone[i].y1;
    		dafs_box.w = zone[i].x2 - zone[i].x1 + 1;
    		dafs_box.h = zone[i].y2 - zone[i].y1 + 1;

    		i_SetBox(dafs_zone, dafs_box);
  	}
  	dafs_error = i_ReadImage(tif_file, &imPtr);
  	i_SetImage(dafs_doc, imPtr);
  	dafs_error=i_WriteEntity(dafs_file, dafs_doc, DAFSB);
  	i_DisposeImage(imPtr);
  	i_DisposeEntity(dafs_doc);
  	i_ExitLib();
}

/* > */
