/*
**  File: EvalDocstrum.c
**  Purpose: compute the objective function value for Docstrum algorithm
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: EvalDocstrum.c,v 1.3 2000/09/20 15:24:00 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "rafport.h"
#include <sys/types.h>
#include <sys/uio.h>
#include "dafstype.h"
#include "rlimage.h"
#include "dafs.h"

#include "pset.h"

void EvalDocstrum(TEST_PROTOCOL *test_protocol, void *docstrum_param,
		       void *bench_param, char *weight_file, char **test_report_p)
{
	FILE *fp;
	char *seg_dafs;
	char *grt_dafs;
	char *tif_file;
	char *cc_file;
	int i;
	float avg_score = 0.0;

  	/* for sh_c */
  	char *sh_command;
  	char *standardout;
  	const int standardoutmax = 256; 
  	char *standarderr;
  	const int standarderrmax = 256;

  	static clock_t time_s, time_e;

	ERR_MEASURE   *err_measure;
	SCORE		*score;

	/* read weight */
	WEIGHT *weight;
	ReadWeight(weight_file, &weight);

	err_measure = (ERR_MEASURE *) malloc (sizeof(ERR_MEASURE));
	score       = (SCORE *) malloc (sizeof(SCORE));

  	/* for sh_c */
  	standardout =  (char *) malloc (256 * sizeof(char));
  	standarderr = (char *) malloc (256 * sizeof(char));

  	seg_dafs = (char *) malloc (256 * sizeof(char));
  	grt_dafs = (char *) malloc (256 * sizeof(char));
  	tif_file = (char *) malloc (256 * sizeof(char));
  	cc_file  = (char *) malloc (256 * sizeof(char));

        switch(test_protocol->ben_alg) {
                case TEXTLINE_BASED:
                        fp = fopen((*test_report_p), "a");
                        fprintf(fp,"# Img\tnSpl\tnMrg\tnFA\tnSplL\tnMrgL\tnMisL\tnErrL\tnGtL\t");
                        fprintf(fp,"score\ttiming\n");
                        break;
                default:
                        printf("No such benchmarking algorithm.\n");
			exit(-1);
                        break;
        }


  	if( 	((DOCSTRUM_PARAM *)(docstrum_param))->tcc < 0 ||
		((DOCSTRUM_PARAM *)(docstrum_param))->tcc > 100 || 
		((DOCSTRUM_PARAM *)(docstrum_param))->par < 0 || 
		((DOCSTRUM_PARAM *)(docstrum_param))->par > 100 || 
  	 	((DOCSTRUM_PARAM *)(docstrum_param))->per < 0 ||
		((DOCSTRUM_PARAM *)(docstrum_param))->per > 100 || 
		((DOCSTRUM_PARAM *)(docstrum_param))->dis < 0 || 
		((DOCSTRUM_PARAM *)(docstrum_param))->dis > 100) { 
    		free(seg_dafs);
    		free(grt_dafs);
    		free(tif_file);
		free(cc_file);
  		free(standardout);
  		free(standarderr);
		free(score);
		free(err_measure);
                fprintf(fp,"Parameter values out of reasonable range.\n");
		fclose(fp);
                exit(-1);

  	}
  	else {
    		for(i = 0; i < test_protocol->dataset->nImg; i++) {
      			time_s = clock();
    			sprintf(seg_dafs,"%s%s%s", test_protocol->test_result_dir,
				test_protocol->dataset->ImgName[i],
				test_protocol->sg_suffix);
    			sprintf(grt_dafs, "%s%s%s", test_protocol->gt_dir,
				test_protocol->dataset->ImgName[i],
				test_protocol->gt_suffix);
    			sprintf(tif_file, "%s%s%s", test_protocol->img_dir,
				test_protocol->dataset->ImgName[i],
				test_protocol->img_suffix);
			sprintf(cc_file, "%s%s%s",
				((DOCSTRUM_PARAM *)(docstrum_param))->concom_dir,
				test_protocol->dataset->ImgName[i],
                                ((DOCSTRUM_PARAM *)(docstrum_param))->concom_suffix);
			if(((DOCSTRUM_PARAM *)(docstrum_param))->alg_mode == FUNC_CALL)
    				DocstrumFunction(tif_file, cc_file,
					((DOCSTRUM_PARAM *)(docstrum_param)), seg_dafs); 
    			else if(((DOCSTRUM_PARAM *)(docstrum_param))->alg_mode == SHELL_CALL) {
      				sh_command = (char *) malloc (256 * sizeof(const char));
      				sprintf(sh_command, "Docstrum.sh %f %f %f %f %s %s %s",
					((DOCSTRUM_PARAM *)(docstrum_param))->tcc,
					((DOCSTRUM_PARAM *)(docstrum_param))->par,
					((DOCSTRUM_PARAM *)(docstrum_param))->per,
					((DOCSTRUM_PARAM *)(docstrum_param))->dis,
					tif_file, cc_file, seg_dafs);
      				sh_c(sh_command, standardout, standardoutmax,
					standarderr, standarderrmax);
      				free(sh_command);
    			}

                        switch(test_protocol->ben_alg) {
                                case TEXTLINE_BASED:
                                        Bench(test_protocol->dataset->ImgName[i],
						(BENCH_PARAM *)bench_param, grt_dafs,
                                                seg_dafs, &err_measure);
                                        score->err_rate = BenchScoring((BENCH_PARAM *)bench_param,
							  err_measure, weight);
					avg_score += score->err_rate;
                                        time_e = clock();
                                        score->timing = ((double)(time_e - time_s))/CLOCKS_PER_SEC;
                                        fprintf(fp, "%s\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%d\t%.3f\t%.3f\n",
                                                test_protocol->dataset->ImgName[i],
                                                err_measure->nSpl,
                                                err_measure->nMrg,
                                                err_measure->nFA,
                                                err_measure->nSplLine,
                                                err_measure->nMrgLine,
                                                err_measure->nMisLine,
                                                err_measure->nErrLine,
                                                err_measure->nGtLine,
                                                score->err_rate,
                                                score->timing);
                                        break;
                                default:
                                        printf("No such benchmarking algorithm.\n");
                                        exit(-1);
                                        break;
                        }

    		}

		avg_score = avg_score / (test_protocol->dataset->nImg);
		avg_score = 1 - avg_score;
		fprintf(fp, "\nThe average textline accuracy = %f\n", avg_score);
		fprintf(fp, "\n# End of testing.\n");

    		free(seg_dafs);
    		free(grt_dafs);
    		free(tif_file);
		free(cc_file);
		free(standardout);
		free(standarderr);
		free(score);
		free(err_measure);
		free(weight);
		fclose(fp);
  	}
} 
