/* 
**  File: FreeJobTrain.c
**  Purpose: free previously allocated memory
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: FreeJobTrain.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdlib.h>
#include <stdio.h>

#include "pset.h"

void FreeJobTrain(TRAIN_PROTOCOL *train_protocol, char *train_report,
		  char *opt_seg_param,
		  char *train_protocol_file, void *opt_param, void *ben_param,
		  void *seg_param, char *opt_file, char *seg_file,
		  char *ben_file, char *weight_file)
{
	int i;

	switch(train_protocol->seg_alg) {
		case XYCUT:
			free(((XYCUT_PARAM *)(seg_param)));
			break;
		case DOCSTRUM:
			free(((DOCSTRUM_PARAM *)(seg_param))->concom_dir);
			free(((DOCSTRUM_PARAM *)(seg_param))->concom_suffix);
			free(((DOCSTRUM_PARAM *)(seg_param)));
			break;
		case VORONOI:
			free(((VORONOI_PARAM *)(seg_param)));
			break;
		default:
			break;
	}

        switch(train_protocol->opt_alg) {
                case SIMPLEX:
 		       	for (i = 1; i <= (((SIMPLEX_PARAM *)(opt_param))->ndim + 1); i++)
          		free(((SIMPLEX_PARAM *)(opt_param))->p[i]);
 		        free(++((SIMPLEX_PARAM *)(opt_param))->p);
		        free(((SIMPLEX_PARAM *)(opt_param))->y);
		        free(((SIMPLEX_PARAM *)(opt_param))->t);
		        free(((SIMPLEX_PARAM *)(opt_param))->scale);
		        free(((SIMPLEX_PARAM *)(opt_param)));
                        break;
                default:
                        break;
        }

        switch(train_protocol->ben_alg) {
                case TEXTLINE_BASED:
			free(((BENCH_PARAM *)(ben_param)));
			break;
                default:
                        break;
        }

	for(i = 0; i < train_protocol->dataset->nImg; i++)
                free(train_protocol->dataset->ImgName[i]);
        free(train_protocol->dataset->ImgName);
	free(train_protocol->dataset);
	free(train_protocol->gt_dir);
	free(train_protocol->img_dir);
	free(train_protocol->gt_suffix);
	free(train_protocol->sg_suffix);
	free(train_protocol->img_suffix);
	free(train_protocol->train_result_dir);

	free(train_protocol);
	free(train_report);
	free(opt_seg_param);
	free(train_protocol_file);
	free(opt_file);
	free(seg_file);
	free(ben_file);
	free(weight_file);
}
