/*
**  File: GenscoreVoronoi.c
**  Purpose: compute the objective function value of X-Y cut algorithm
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: GenscoreVoronoi.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "rafport.h"
#include <sys/types.h>
#include <sys/uio.h>
#include "dafstype.h"
#include "rlimage.h"
#include "dafs.h"

#include "pset.h"


void GenscoreVoronoi(TRAIN_PROTOCOL *train_protocol, void *voronoi_param,
		     void *bench_param, char *weight_file, float *val, SCORE **score_p)
{
	char *seg_dafs;
	char *grt_dafs;
	float avg_score = 0;
	char *tif_file;
	int i;

	ERR_MEASURE     *err_measure;
	WEIGHT		*weight;

  	/* for sh_c */
  	char *sh_command;
  	char *standardout;
  	const int standardoutmax = 256; 
  	char *standarderr;
  	const int standarderrmax = 256;

  	static clock_t time_s, time_e;
  	static clock_t time_l_s, time_l_e;

	/* read weight */
	ReadWeight(weight_file, &weight);

        /* timing information */
        time_s = clock();

	err_measure = (ERR_MEASURE *) malloc (sizeof(ERR_MEASURE));

  	/* for sh_c */
  	standardout =  (char *) malloc (256 * sizeof(char));
  	standarderr = (char *) malloc (256 * sizeof(char));

  	((VORONOI_PARAM *)(voronoi_param))->sr = (int) val[1];
  	((VORONOI_PARAM *)(voronoi_param))->nm = (int) val[2];    
  	((VORONOI_PARAM *)(voronoi_param))->fr = (float)     val[3];
  	((VORONOI_PARAM *)(voronoi_param))->ta = (int) val[4];

  	printf("para = (%d,%d,%f,%d)\n", ((VORONOI_PARAM *)(voronoi_param))->sr,
					 ((VORONOI_PARAM *)(voronoi_param))->nm,
                                    	 ((VORONOI_PARAM *)(voronoi_param))->fr, 
					 ((VORONOI_PARAM *)(voronoi_param))->ta );

  	seg_dafs = (char *) malloc (256 * sizeof(char));
  	grt_dafs = (char *) malloc (256 * sizeof(char));
  	tif_file = (char *) malloc (256 * sizeof(char));

  	if(val[1] < 1 || val[1] > 100 || val[2] < 2 || 
	   val[2] > 500 || val[3] < 0.001 || val[3] > 0.999 || 
	   val[4] < 1 || val[4] > 800) {
    		avg_score = 1.0;
    		time_e = clock();
		(*score_p)->err_rate = avg_score;
		(*score_p)->timing = ((double)(time_e - time_s))/CLOCKS_PER_SEC;
  	}
  	else {
    		avg_score = 0.0;
    		for(i = 0; i < train_protocol->dataset->nImg; i++) {
      			time_l_s = clock();
    			sprintf(seg_dafs,"%sjunk%s", train_protocol->train_result_dir,
				train_protocol->sg_suffix);
    			sprintf(grt_dafs, "%s%s%s", train_protocol->gt_dir,
				train_protocol->dataset->ImgName[i],
				train_protocol->gt_suffix);
    			sprintf(tif_file, "%s%s%s", train_protocol->img_dir,
				train_protocol->dataset->ImgName[i],
				train_protocol->img_suffix);
			
			if(((VORONOI_PARAM *)(voronoi_param))->alg_mode == FUNC_CALL)
    				VoronoiFunction(tif_file, (VORONOI_PARAM *) voronoi_param,
					      seg_dafs); 
    			else if(((VORONOI_PARAM *)(voronoi_param))->alg_mode ==
				SHELL_CALL) {
      				sh_command = (char *) malloc (256 * sizeof(const char));
      				sprintf(sh_command, "Voronoi.sh %d %d %f %d %s %s",
					((VORONOI_PARAM *)(voronoi_param))->sr,
					((VORONOI_PARAM *)(voronoi_param))->nm,
					((VORONOI_PARAM *)(voronoi_param))->fr,
					((VORONOI_PARAM *)(voronoi_param))->ta,
					tif_file, seg_dafs);
      				sh_c(sh_command, standardout, standardoutmax,
					standarderr, standarderrmax);
      				free(sh_command);
    			}

			switch(train_protocol->ben_alg) {
				case TEXTLINE_BASED:
    					Bench("junk", (BENCH_PARAM *)bench_param, grt_dafs,
					      seg_dafs, &err_measure);
					(*score_p)->err_rate = BenchScoring((BENCH_PARAM *)bench_param,
									    err_measure, weight); 
					break;
				default:
					printf("No such benchmarking algorithm.\n");
					exit(-1);
					break;
			}


    			time_l_e = clock();
    			printf("%s err_pert = %f\ttiming = %f\n",
				train_protocol->dataset->ImgName[i],
				(*score_p)->err_rate,
				((double)(time_l_e - time_l_s))/CLOCKS_PER_SEC); 
    			fflush(NULL);
    			avg_score = avg_score + (*score_p)->err_rate;

    		}
    		avg_score = avg_score / train_protocol->dataset->nImg;
    		time_e = clock();
		(*score_p)->err_rate = avg_score;
		(*score_p)->timing = ((double)(time_e - time_s))/CLOCKS_PER_SEC;
  	}
	free(seg_dafs);
	free(grt_dafs);
	free(tif_file);
	free(standardout);
	free(standarderr);
	free(err_measure);
	free(weight);
} 
