/* 
**  File: ReadBenchParam.c
**  Purpose: read BenchMarking algorithm parameters
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: ReadBenchParam.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"

void ReadBenchParam(char *bench_file, void **ben_param_p)
{
        FILE *fp;
        char c;
        char *item, *tmp, *itemVal;

        item = (char *) malloc (256 * sizeof(char));
        tmp = (char *) malloc (256 * sizeof(char));
        itemVal = (char *) malloc (256 * sizeof(char));

        ((BENCH_PARAM *)(*ben_param_p)) = (BENCH_PARAM  *) malloc (sizeof(BENCH_PARAM));

        fp = fopen(bench_file, "r");
        if(fp == NULL) {
                printf("Can not open xycut parameter file!\n");
                exit(-1);
        }

        while(!feof(fp)) {
                c = fgetc(fp);
                switch(c) {
                        case '#':
                                fgets(tmp,256,fp);
                                break;
                        case EOF:
                                break;
                        case ' ':
                                break;
                        case '\t':
                                break;
                        case '\n':
                                break;
                        default:
				ungetc(c,fp);
                                fscanf(fp, "%s%s%s", item, tmp, itemVal);
				if(strcmp(item,"HTOL") == 0) {
                                  if(atoi(itemVal) <= 0) {
                                    printf("Invalid Horizontal Tolerance Value.\n");
                                    exit(-1);
                                  }
                                  else 
                                    ((BENCH_PARAM *)(*ben_param_p))->htol = atoi(itemVal);
				}
                                else if(strcmp(item,"VTOL") == 0) {
                                  if(atoi(itemVal) <= 0) {
                                    printf("Invalid Vertical Tolerance Value.\n");
                                    exit(-1);
                                  }
                                  else 
                                    ((BENCH_PARAM *)(*ben_param_p))->vtol = atoi(itemVal);
				}
                                else if(strcmp(item,"HPIX") == 0) {
                                  if(atoi(itemVal) <= 0) {
                                    printf("Invalid Horizontal Pixels Value.\n");
                                    exit(-1);
                                  }
                                  else 
                                    ((BENCH_PARAM *)(*ben_param_p))->hpix = atoi(itemVal);
				}
                                else if(strcmp(item,"VPIX") == 0) {
                                  if(atoi(itemVal) <= 0) {
                                    printf("Invalid Vertical Pixels Value.\n");
                                    exit(-1);
                                  }
                                  else 
                                    ((BENCH_PARAM *)(*ben_param_p))->vpix = atoi(itemVal);
				}
				else {
				  printf("Invalid benchmarking parameter item.\n");
				  exit(-1);
				}
				break;
		}
	}
	fclose(fp);
	free(item);
	free(tmp);
	free(itemVal);
}
