/* 
**  File: ReadDocstrumParam.c
**  Purpose: read Docstrum segmentation algorithm parameters
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: ReadDocstrumParam.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"

void ReadDocstrumParam(char *docstrum_file, void **docstrum_param_p)
{
        FILE *fp;
        char c;
        char *item, *tmp, *itemVal;

        item = (char *) malloc (256 * sizeof(char));
        tmp = (char *) malloc (256 * sizeof(char));
        itemVal = (char *) malloc (256 * sizeof(char));

        ((DOCSTRUM_PARAM *)(*docstrum_param_p)) = (DOCSTRUM_PARAM  *) malloc (sizeof(DOCSTRUM_PARAM));

        fp = fopen(docstrum_file, "r");
        if(fp == NULL) {
                printf("Can not open docstrum parameter file!\n");
                exit(-1);
        }

        while(!feof(fp)) {
                c = fgetc(fp);
                switch(c) {
                        case '#':
                                fgets(tmp,256,fp);
                                break;
                        case EOF:
                                break;
                        case ' ':
                                break;
                        case '\t':
                                break;
                        case '\n':
                                break;
                        default:
				ungetc(c,fp);
				fscanf(fp, "%s%s%s", item, tmp, itemVal);
                                if(strcmp(item, "ALG_MODE") == 0) {
                                  if(strcmp(itemVal, "func_call") == 0)
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->alg_mode = FUNC_CALL;
                                  else if(strcmp(itemVal, "shell_call") == 0)
                                     ((DOCSTRUM_PARAM *)(*docstrum_param_p))->alg_mode = SHELL_CALL;
                                  else {
                                    printf("No such algorithm call mode.\n");
                                    exit(-1);
                                  }
                                }
                                else if(strcmp(item,"CONCOM_DIR") == 0) {
                                  ((DOCSTRUM_PARAM *)(*docstrum_param_p))->concom_dir = (char *) malloc (256 * sizeof(char));
                                  sscanf(itemVal, "%s", ((DOCSTRUM_PARAM *)(*docstrum_param_p))->concom_dir);
                                }
                                else if(strcmp(item,"CONCOM_SUFFIX") == 0) {
                                  ((DOCSTRUM_PARAM *)(*docstrum_param_p))->concom_suffix = (char *) malloc (256 * sizeof(char));
                                  sscanf(itemVal, "%s", ((DOCSTRUM_PARAM *)(*docstrum_param_p))->concom_suffix);
                                }
				else if(strcmp(item, "KNN") == 0) {
				  if(atoi(itemVal) <= 0) {
                                    printf("Invalid K nearest neighbour clustering threshold.\n");
                                    exit(-1);
                                  }
                                  else 
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->knn = atoi(itemVal);
				}
				else if(strcmp(item, "LOWSIZE") == 0) {
                                  if(atoi(itemVal) <= 0) {    
                                    printf("Invalid size lowerbound threshold.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->lsize = atoi(itemVal);
				}
				else if(strcmp(item, "HIGHSIZE") == 0) {
                                  if(atoi(itemVal) <= 0) {    
                                    printf("Invalid size upperbound threhols.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->hsize = atoi(itemVal);
				}
				else if(strcmp(item, "HANGTOL") == 0) {
                                  if(atoi(itemVal) <= 0) {    
                                    printf("Invalid horizontal angle tolerance.\n");
                                    exit(-1);  
                                  }    
                                  else 
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->hatol = atoi(itemVal);
				}
                                else if(strcmp(item, "VANGTOL") == 0) {
                                  if(atoi(itemVal) <= 0) {
                                    printf("Invalid vertical angle tolerance.\n");
                                    exit(-1); 
                                  }   
                                  else
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->vatol = atoi(itemVal);
				}
                                else if(strcmp(item, "TCC") == 0) {
                                  if(atof(itemVal) <= 0) {  
                                    printf("Invalid CC distance threhols.\n");
                                    exit(-1); 
                                  }   
                                  else
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->tcc = atof(itemVal);
				}
                                else if(strcmp(item, "PAR") == 0) {
                                  if(atof(itemVal) <= 0) {   
                                    printf("Invalid parallel distance threshold.\n");
                                    exit(-1); 
                                  }   
                                  else
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->par = atof(itemVal);
				}
                                else if(strcmp(item, "PER") == 0) {
                                  if(atof(itemVal) <= 0) {
                                    printf("Invalid perpendicular distance threshold.\n");
                                    exit(-1); 
                                  }   
                                  else
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->per = atof(itemVal);
				}
                                else if(strcmp(item, "SCR") == 0) {
                                  if(atof(itemVal) <= 0) {  
                                    printf("Invalid super/sub script distance threhols.\n");
                                    exit(-1); 
                                  }   
                                  else
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->scr = atof(itemVal);
				}
                                else if(strcmp(item, "DIS") == 0) {
                                  if(atof(itemVal) <= 0) {   
                                    printf("Invalid distance threshold.\n");
                                    exit(-1); 
                                  }   
                                  else
                                    ((DOCSTRUM_PARAM *)(*docstrum_param_p))->dis = atof(itemVal);
				}
				else {
				  printf("Invalid Docstrum algorithm parameter item.\n");
				  exit(-1);
				}
				break;
		}
	}
	fclose(fp);
	free(item);
	free(tmp);
	free(itemVal);
}
