/* 
**  File: ReadSimplexParam.c
**  Purpose: read Simplex optimization algorithm parameters
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: ReadSimplexParam.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "pset.h"

void ReadSimplexParam(char *simplex_file, void **simplex_param_p)
{
	FILE *fp;
        char c;
	int  i;
        char *item, *tmp, *itemVal, *data;
	char ndimflg = OFF;

        item = (char *) malloc (256 * sizeof(char));
        tmp = (char *) malloc (256 * sizeof(char));
        itemVal = (char *) malloc (256 * sizeof(char));

	(SIMPLEX_PARAM *) ((SIMPLEX_PARAM *)(*simplex_param_p)) = (SIMPLEX_PARAM *) malloc (sizeof(SIMPLEX_PARAM));

        fp = fopen(simplex_file, "r");
        if(fp == NULL) {
                printf("Can not open simplex parameter file!\n");
                exit(-1);
        }

        while(!feof(fp)) {
                c = fgetc(fp);
                switch(c) {
                        case '#':
                                fgets(tmp,256,fp);
                                break;
                        case EOF:
                                break;
                        case ' ':
                                break;
			case '\t':
				break;
                        case '\n':
                                break;
                        default:
				ungetc(c,fp);
                                fscanf(fp, "%s%s%s", item, tmp, itemVal);
				if(strcmp(item,"NDIM") == 0) {
				  	if(atoi(itemVal) <= 0) {
			            		printf("Invalid dimension number.\n");
				    		exit(-1);
			  	  	} 
				  	else {
						ndimflg = ON;
				    		((SIMPLEX_PARAM *)(*simplex_param_p))->ndim = atoi(itemVal);
				    		((SIMPLEX_PARAM *)(*simplex_param_p))->p =
							(float **) malloc ( (((SIMPLEX_PARAM *)
							(*simplex_param_p))->ndim + 1) * sizeof(float *) );
 				    		(((SIMPLEX_PARAM *)(*simplex_param_p))->p)--;
        			    		for(i = 1; i <= (atoi(itemVal) + 1); i++) {
           			    			((SIMPLEX_PARAM *)(*simplex_param_p))->p[i] =
								(float *) malloc ( (atoi(itemVal) + 1) *
								sizeof(float));
        			    		}
				    		((SIMPLEX_PARAM *)(*simplex_param_p))->y =
							(float *) malloc ( (atoi(itemVal) + 2) * sizeof(float) );
				    		((SIMPLEX_PARAM *)(*simplex_param_p))->t =
							(float *) malloc ( (atoi(itemVal) + 2) * sizeof(float) );
				    		((SIMPLEX_PARAM *)(*simplex_param_p))->scale= (float *) malloc
							( (atoi(itemVal) + 2) * sizeof(float) );
				 	}
				}
                                else if(strcmp(item,"CRIFLG") == 0) {
                                  	if(strcmp(itemVal,"nelder-mead") == 0)
                                    		((SIMPLEX_PARAM *)(*simplex_param_p))->criflg = NM;
                                  	else if(strcmp(itemVal,"numerical-recipe") == 0)
                                    		((SIMPLEX_PARAM *)(*simplex_param_p))->criflg = NR;
                                  	else {
                                    		printf("No such stop criterion exists.\n");
                                    		exit(-1);
                                  	}
				}
				else if(strcmp(item,"NMAX") == 0) {
				  	if(atoi(itemVal) <= 0) { 
                                    		printf("Invalid function evaluation number upper bound.\n");
                                    		exit(-1);
                                  	} 
                                  	else
                                    		((SIMPLEX_PARAM *)(*simplex_param_p))->nmax = atoi(itemVal);
				}
			        else if(strcmp(item,"FTOL") == 0) {
                                  	if(atof(itemVal) <= 0) { 
                                    		printf("Invalid function tolarence value.\n");
                                    		exit(-1);
                                  	} 
                                  	else
                                    		((SIMPLEX_PARAM *)(*simplex_param_p))->ftol = atof(itemVal);
				}
				else if(strcmp(item,"ALPHA") == 0) {
                                  	if(atof(itemVal) <= 0) { 
                                    		printf("Invalid alpha value.\n");
                                    		exit(-1);
                                  	} 
                                  	else
                                    		((SIMPLEX_PARAM *)(*simplex_param_p))->alpha = atof(itemVal);
				}
				else if(strcmp(item,"BETA") == 0) {
                                  	if(atof(itemVal) <= 0) { 
                                    		printf("Invalid beta value.\n");
                                    		exit(-1);
                                  	} 
                                  	else
                                    		((SIMPLEX_PARAM *)(*simplex_param_p))->beta = atof(itemVal);
				}
				else if(strcmp(item,"GAMMA") == 0) {
                                  	if(atof(itemVal) <= 0) { 
                                    		printf("Invalid gamma value.\n");
                                    		exit(-1);
                                  	} 
                                  	else
                                    		((SIMPLEX_PARAM *)(*simplex_param_p))->gamma = atof(itemVal);
				}
				else if(strcmp(item,"SIGMA") == 0) {
                                  	if(atof(itemVal) <= 0) { 
                                    		printf("Invalid sigma value.\n");
                                    		exit(-1);
                                  	} 
                                  	else
                                    		((SIMPLEX_PARAM *)(*simplex_param_p))->sigma = atof(itemVal);
				}
                                else if(strcmp(item,"P") == 0) {
				  if(ndimflg == ON) {
				    data = strtok(itemVal, ",");
				    sscanf(data,"%f", &((((SIMPLEX_PARAM *)(*simplex_param_p))->p)[1][1]));
				    if(((SIMPLEX_PARAM *)(*simplex_param_p))->ndim > 1) {
				      for(i = 2; i <= ((SIMPLEX_PARAM *)(*simplex_param_p))->ndim; i++) { 
				        data = strtok(NULL, ",");
				        sscanf(data,"%f", &((((SIMPLEX_PARAM *)(*simplex_param_p))->p)[1][i])); 
				      }
				    }
				  }
				}
                                else if(strcmp(item,"SCALE") == 0) {
				  if(ndimflg == ON) {
				    data = strtok(itemVal, ",");
				    sscanf(data,"%f", &(((SIMPLEX_PARAM *)(*simplex_param_p))->scale[1]));
				    if(((SIMPLEX_PARAM *)(*simplex_param_p))->ndim > 1) {
				      for(i = 2; i <= ((SIMPLEX_PARAM *)(*simplex_param_p))->ndim; i++) { 
				        data = strtok(NULL, ",");
				        sscanf(data,"%f", &(((SIMPLEX_PARAM *)(*simplex_param_p))->scale[i])); 
				      }
				    }
				  }
				}
				else {
				  printf("Invalid simplex algorithm parameter item.\n");
				  exit(-1);
				}
				break;
		}	
	}
	fclose(fp);
	free(item);
	free(tmp);
	free(itemVal);
}
