/* 
**  File: ReadTrainProtocol.c
**  Purpose: read the training protocol
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: ReadTrainProtocol.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"

void ReadTrainProtocol(char *train_protocol_file, TRAIN_PROTOCOL **train_protocol_p)
{
	FILE *fp;
	char c, segflg;
	char *item, *tmp, *itemVal;

	item = (char *) malloc (256 * sizeof(char));
	tmp = (char *) malloc (256 * sizeof(char));
	itemVal = (char *) malloc (256 * sizeof(char));

	(*train_protocol_p) 	= (TRAIN_PROTOCOL *) malloc (sizeof(TRAIN_PROTOCOL));

	fp = fopen(train_protocol_file, "r");
	if(fp == NULL) {
		printf("Can not open protocol file!\n");
		exit(-1);
	}

	segflg = 0;
	while(!feof(fp)) {
		c = fgetc(fp);
		switch(c) {
			case '#':
				fgets(tmp,256,fp);
				break;
			case EOF:
				break;
			case ' ':
				break;
                        case '\t':
                                break;
			case '\n':
				break;
			default:
				ungetc(c,fp);
				fscanf(fp, "%s%s%s", item, tmp, itemVal);
				if(strcmp(item,"DATASET") == 0) {
				  (*train_protocol_p)->dataset = (DATASET *) malloc (sizeof(DATASET));
				  ReadFileNames(itemVal, &((*train_protocol_p)->dataset));
				}
				else if(strcmp(item,"GROUNDTRUTH_DIR") == 0) {
				  (*train_protocol_p)->gt_dir = (char *) malloc (256 * sizeof(char));
				  sscanf(itemVal, "%s", (*train_protocol_p)->gt_dir);
				}
                                else if(strcmp(item,"IMG_DIR") == 0) {
                                  (*train_protocol_p)->img_dir = (char *) malloc (256 * sizeof(char));
                                  sscanf(itemVal, "%s", (*train_protocol_p)->img_dir);
                                }
                                else if(strcmp(item,"GT_SUFFIX") == 0) {
                                  (*train_protocol_p)->gt_suffix = (char *) malloc (256 * sizeof(char));
                                  sscanf(itemVal, "%s", (*train_protocol_p)->gt_suffix);
                                }
                                else if(strcmp(item,"SG_SUFFIX") == 0) {
                                  (*train_protocol_p)->sg_suffix = (char *) malloc (256 * sizeof(char));
                                  sscanf(itemVal, "%s", (*train_protocol_p)->sg_suffix);
                                }
                                else if(strcmp(item,"IMG_SUFFIX") == 0) {
                                  (*train_protocol_p)->img_suffix = (char *) malloc (256 * sizeof(char));
                                  sscanf(itemVal, "%s", (*train_protocol_p)->img_suffix);
                                }
                                else if(strcmp(item,"TRAIN_RESULT_DIR") == 0) {
                                  (*train_protocol_p)->train_result_dir = (char *) malloc (256 * sizeof(char));
                                  sscanf(itemVal, "%s", (*train_protocol_p)->train_result_dir);
                                }
				else if(strcmp(item,"OPT_ALG") == 0) {
				  if(strcmp(itemVal,"simplex") == 0) 
				    (*train_protocol_p)->opt_alg = SIMPLEX;
				  else {
				    printf("No such optimization algorithm avalible.\n");
				    exit(-1);
				  }
				}
                                else if(strcmp(item,"BEN_ALG") == 0) {
                                  if(strcmp(itemVal,"textline_based") == 0)
                                    (*train_protocol_p)->ben_alg = TEXTLINE_BASED;
                                  else {
                                    printf("No such benchmarking algorithm avalible.\n");
                                    exit(-1);
                                  }
				}
                                else if(strcmp(item,"SEG_ALG") == 0) {
				  segflg = 1;
                                  if(strcmp(itemVal,"xycut") == 0)
				    (*train_protocol_p)->seg_alg = XYCUT;
				  else if(strcmp(itemVal,"docstrum") == 0)
				    (*train_protocol_p)->seg_alg = DOCSTRUM;
				  else if(strcmp(itemVal,"voronoi") == 0) 
				    (*train_protocol_p)->seg_alg = VORONOI;
				  else if(strcmp(itemVal,"test_driver") == 0) 
				    (*train_protocol_p)->seg_alg = TEST_DRIVER;
                                  else {
				    segflg = 0;
                                    printf("No such segmentation algorithm avalible.\n");
                                    exit(-1);
                                  }
				}
				else {
				  printf("Invalid protocol parameter item.\n");
				  exit(-1);
				}
			        break;
		}
	}
	fclose(fp);
	free(item);
	free(tmp);
	free(itemVal);

}
