/* 
**  File: ReadVoronoiParam.c
**  Purpose: read X-Y cut segmentation algorithm parameters
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: ReadVoronoiParam.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"

void ReadVoronoiParam(char *voronoi_file, void **voronoi_param_p)
{
        FILE *fp;
        char c;
        char *item, *tmp, *itemVal;

        item = (char *) malloc (256 * sizeof(char));
        tmp = (char *) malloc (256 * sizeof(char));
        itemVal = (char *) malloc (256 * sizeof(char));

	(VORONOI_PARAM *)(*voronoi_param_p) = (VORONOI_PARAM *) malloc (sizeof(VORONOI_PARAM));

        fp = fopen(voronoi_file, "r");
        if(fp == NULL) {
                printf("Can not open voronoi parameter file!\n");
                exit(-1);
        }

        while(!feof(fp)) {
                c = fgetc(fp);
                switch(c) {
                        case '#':
                                fgets(tmp,256,fp);
                                break;
                        case EOF:
                                break;
                        case ' ':
                                break;
                        case '\t':
                                break;
                        case '\n':
                                break;
                        default:
				ungetc(c,fp);
                                fscanf(fp, "%s%s%s", item, tmp, itemVal);
                                if(strcmp(item, "ALG_MODE") == 0) {
                                  if(strcmp(itemVal, "func_call") == 0)
                                    ((VORONOI_PARAM *)(*voronoi_param_p))->alg_mode = FUNC_CALL;
                                  else if(strcmp(itemVal, "shell_call") == 0)
                                     ((VORONOI_PARAM *)(*voronoi_param_p))->alg_mode = SHELL_CALL;
                                  else {
                                    printf("No such algorithm call mode.\n");
                                    exit(-1);
                                  }
                                }
				else if(strcmp(item, "SR") == 0) {
				  if(atoi(itemVal) <= 0) {
                                    printf("Invalid sampling rate value.\n");
                                    exit(-1);
                                  }
                                  else 
                                    ((VORONOI_PARAM *)(*voronoi_param_p))->sr = atoi(itemVal);
				}
				else if(strcmp(item, "NM") == 0) {
                                  if(atoi(itemVal) <= 0) {    
                                    printf("Invalid maximum noise block size number.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((VORONOI_PARAM *)(*voronoi_param_p))->nm = atoi(itemVal);
				}
				else if(strcmp(item, "FR") == 0) {
                                  if(atof(itemVal) <= 0) {    
                                    printf("Invalid gap ratio number.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((VORONOI_PARAM *)(*voronoi_param_p))->fr = atof(itemVal);
				}
				else if(strcmp(item, "TA") == 0) {
                                  if(atoi(itemVal) <= 0) {    
                                    printf("Invalid connected component area ratio threshold.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((VORONOI_PARAM *)(*voronoi_param_p))->ta = atoi(itemVal);
				}
				else if(strcmp(item, "SW") == 0) {
                                  if(atof(itemVal) <= 0) {    
                                    printf("Invalid connected component area ratio threshold.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((VORONOI_PARAM *)(*voronoi_param_p))->sw = atof(itemVal);
				}
				else {
				  printf("Invalid Voronoi-based algorithm parameter item.\n");
				  exit(-1);
				}
				break;
		}
	}
	fclose(fp);
	free(item);
	free(tmp);
	free(itemVal);
}
