/* 
**  File: ReadXycutParam.c
**  Purpose: read X-Y cut segmentation algorithm parameters
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: ReadXycutParam.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"

void ReadXycutParam(char *xycut_file, void **xycut_param_p)
{
        FILE *fp;
        char c;
        char *item, *tmp, *itemVal;

        item = (char *) malloc (256 * sizeof(char));
        tmp = (char *) malloc (256 * sizeof(char));
        itemVal = (char *) malloc (256 * sizeof(char));

	((XYCUT_PARAM *)(*xycut_param_p)) = (XYCUT_PARAM *) malloc (sizeof(XYCUT_PARAM));

        fp = fopen(xycut_file, "r");
        if(fp == NULL) {
                printf("Can not open xycut parameter file!\n");
                exit(-1);
        }

        while(!feof(fp)) {
                c = fgetc(fp);
                switch(c) {
                        case '#':
                                fgets(tmp,256,fp);
                                break;
                        case EOF:
                                break;
                        case ' ':
                                break;
                        case '\t':
                                break;
                        case '\n':
                                break;
                        default:
				ungetc(c,fp);
                                fscanf(fp, "%s%s%s", item, tmp, itemVal);
                                if(strcmp(item, "ALG_MODE") == 0) {
                                  if(strcmp(itemVal, "func_call") == 0)
                                    ((XYCUT_PARAM *)(*xycut_param_p))->alg_mode = FUNC_CALL;
                                  else if(strcmp(itemVal, "shell_call") == 0)
                                     ((XYCUT_PARAM *)(*xycut_param_p))->alg_mode = SHELL_CALL;
                                  else {
                                    printf("No such algorithm call mode.\n");
                                    exit(-1);
                                  }
                                }
				else if(strcmp(item, "TNX") == 0) {
				  if(atoi(itemVal) <= 0) {
                                    printf("Invalid X noise removal threshold.\n");
                                    exit(-1);
                                  }
                                  else 
                                    ((XYCUT_PARAM *)(*xycut_param_p))->Tnx = atoi(itemVal);
				}
				else if(strcmp(item, "TNY") == 0) {
                                  if(atoi(itemVal) <= 0) {    
                                    printf("Invalid Y noise removal threshold.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((XYCUT_PARAM *)(*xycut_param_p))->Tny = atoi(itemVal);
				}
				else if(strcmp(item, "TCX") == 0) {
                                  if(atoi(itemVal) <= 0) {    
                                    printf("Invalid X cut width threhols.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((XYCUT_PARAM *)(*xycut_param_p))->Tcx = atoi(itemVal);
				}
				else if(strcmp(item, "TCY") == 0) {
                                  if(atoi(itemVal) <= 0) {    
                                    printf("Invalid Y cut width threshold.\n");     
                                    exit(-1);  
                                  }    
                                  else 
                                    ((XYCUT_PARAM *)(*xycut_param_p))->Tcy = atoi(itemVal);
				}
				else {
				  printf("Invalid X-Y cut algorithm parameter item.\n");
				  exit(-1);
				}
				break;
		}
	}
	fclose(fp);
	free(item);
	free(tmp);
	free(itemVal);
}
