/*
**  File: Train.c
**  Purpose: points to the objective function to be optimized
**	     and call the optimization algorithm
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: Train.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"

void Train(TRAIN_PROTOCOL *train_protocol, void *opt_param, void *ben_param,
	   void *seg_param, char *weight_file, char **train_report_p, char **opt_seg_param_p)
{
	/* pointer to the objective function to be optimized */
        void (*genscore)();

        switch (train_protocol->seg_alg) {
                case XYCUT:
                        genscore = GenscoreXycut;  	/* link to X-Y cut objective function */
                        break;
                case DOCSTRUM:
                        genscore = GenscoreDocstrum;   	/* link to Docstrum objective function */
                        break;
                case VORONOI:
                        genscore = GenscoreVoronoi;    	/* link to Docstrum objective function */
                        break;
                case TEST_DRIVER:
                        genscore = GenscoreTestDriver;  /* test objective function for optimization */
                        break;
                default:
			genscore = NULL;
			printf("No such page segmentation algorithm. Exiting ...\n");
			exit(-1);
                        break;
        }

	switch(train_protocol->opt_alg) {
		case SIMPLEX:
			Simplex(train_protocol, opt_param, ben_param, seg_param,
				weight_file, genscore, train_report_p, opt_seg_param_p);  /* Nelder-Mead simplex algorithm */ 
			break;
		/*
		case GENERIC:
			Ga(train_protocol, opt_param, ben_param,	
				seg_param, genscore, train_report_p);
			break;
		case SIMULATED_ANNEALING:
			Sa(train_protocol, opt_param, ben_param,       
                                seg_param, genscore, train_report_p);
                        break;
		*/

		default:
			printf("No such optimization algorithm.\n");
 	       		exit(-1);
			break;
	}
}
