/*
**  File: WriteTrainReportHeader.c
**  Purpose: write the header comment for training/testing report
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: WriteTrainReportHeader.c,v 1.1 2000/09/19 22:16:27 maosong Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/utsname.h>

#include "pset.h"

void WriteTrainReportHeader(int argc, char **argv, TRAIN_PROTOCOL *train_protocol, char **file_p)
{      
	FILE *fp;
        char *cal_time, *PSETuser;
        static struct utsname *info;
        int errflg;
	char optName[256];
	char segName[256];
	char *wdir, *tmp;
	int i;

	fp = fopen(*file_p, "w");

        fprintf(fp,"#\n");
        fprintf(fp,"# File: %s\n", *file_p);

	switch (train_protocol->opt_alg) {
		case SIMPLEX:
	  		sscanf("Simplex", "%s", optName);
			break;
		default:
			printf("No such optimization algorithm.\n");
			exit(-1);
			break;
	}

        switch(train_protocol->seg_alg) {
                case XYCUT:
                        strcpy(segName, "X-Y cut");
                        break;
                case DOCSTRUM:
                        strcpy(segName, "Docstrum");
                        break;
                case VORONOI:
                        strcpy(segName, "Voronoi-based");
                        break;
                case TEST_DRIVER:
                        strcpy(segName, "Test_driver");
                        break;
                default:
                        printf("No such segmentation algorithm.\n");
                        exit(-1);
                        break;
        }

        fprintf(fp,"# Purpose: training result of the %s algorithm using %s algorithm.\n", segName, optName);
        PSETuser = cuserid(NULL);
	if(PSETuser == NULL)
		fprintf(fp, "Can not find login name.\n");
	else
         	fprintf(fp,"# User: %s\n", PSETuser);
        cal_time = CalendarTime();
        fprintf(fp,"# Date: %s\n", cal_time);
        free(cal_time);
        info = (struct utsname *) malloc (sizeof(struct utsname));
        errflg = uname(info);
        if(errflg == -1) {
                printf("Can not get user and system information.\n");
                exit(-1);
        }
        fprintf(fp,"# Operating system: %s, %s, %s\n", info->sysname,  info->release, info->version);
        fprintf(fp,"# Machine name: %s\n", info->nodename);
	wdir = (char *) malloc (256 * sizeof(char));
	tmp = getcwd(wdir, 256);
        fprintf(fp,"# Working directory: %s\n", wdir);
	free(wdir);
        fprintf(fp,"# Machine type: %s\n", info->machine);
        free(info);
	fprintf(fp,"# Command line: ");
	for(i = 0; i < argc; i++)
		fprintf(fp, "%s ", argv[i]);
        fprintf(fp,"\n");
        fprintf(fp,"#\n");
	fclose(fp);
}
