#ifdef RECT
/* static char version_string[]="$Id: brect.c,v 1.1 2000/09/19 22:16:27 maosong Exp $"; */
/*
   $Date: 2000/09/19 22:16:27 $
   $Revision: 1.1 $
   $Author: maosong $
   brect.c
*/
#include <stdio.h>

#include "pset.h"

static unsigned int current_crect_size=INITCRECT;
static unsigned int current_brect_size=INITBRECT;

/* is_noise_crect 
   if a bounding box "rect" of a connected component
   can be regarded as noise, this function returns YES.
   Otherwise NO. */
int is_noise_crect(Rect rect)
{
  int width;
  int height;
  /*  int area; */
  float aspect;

  width  = rwidth(rect);
  height = rheight(rect);
  /*   area   = rarea(rect); */
  aspect = raspect(rect);

  /* definition of valid connected components */
  if ( width <= crect_width_max &&
       height <= crect_height_max &&
       /* area <= crect_area_max && */
       aspect <= crect_aspect_max ){
    return NO;
  }
  else{
    return YES;
  }
}

int rwidth(Rect rect)
{
  return rect.xe-rect.xs+1;
}

int rheight(Rect rect)
{
  return rect.ye-rect.ys+1;
}

int rarea(Rect rect)
{
  return rwidth(rect)*rheight(rect);
}

int raspect(Rect rect)
{
  int width, height;
  width = rwidth(rect);
  height= rheight(rect);
  return ((width>height)?(width/height):(height/width));
  /* (longer side)/(shorter side) */
}

void init_crect()
{
  int i;
  
  crect = (Rect *)myalloc(sizeof(Rect)*INITCRECT);

  for(i=0;i<INITCRECT;i++)
    crect[i].xs=NORECT;
}

void resize_crect()
{
  int i;

  crect = (Rect *)myrealloc(crect,current_crect_size,INCCRECT,sizeof(Rect));

  for(i=current_crect_size;i<current_crect_size+INCCRECT;i++)
    crect[i].xs=NORECT;

  current_crect_size+=INCCRECT;
}

/* crectset
   to record a bounding box of a connected component whose label is ln
   in "crect" of type "Rect" */
void crectset(Rect rect, Label ln)
{
  if(ln>=current_crect_size){
    if(ln>=LABELMAX) {
      fprintf(stderr,"The number of CC exceeds the limit\n");
      fprintf(stderr,"Change the datatype of Label\n");
      exit(1);
    }
    resize_crect();
  }
  crect[ln]=rect;
}

void generate_brect()
{
  int i;

  init_brect();
  init_cb_tab();

  for(i=0;i<LINEnbr;i++){
    if(lineseg[i].yn == NO_OUTPUT &&
       crect[lineseg[i].lab1].xs != NORECT &&
       crect[lineseg[i].lab2].xs != NORECT){
      /* connected components are in the same segmented block */
      if(BRECTid >= current_brect_size){
	resize_brect();
      }
      set_brect(lineseg[i].lab1,lineseg[i].lab2);
    }
  }
  for(i=0;i<LABELnbr;i++){ /* for blocks with single connected component */
    if(cb_tab[i]==NOCB && crect[i].xs!=NORECT){ /* single and not noise */
      if(BRECTid >= current_brect_size){
	resize_brect();
      }
      brect[BRECTid]=crect[i];
      BRECTid++;
    }
  }
}

void init_brect()
{
  int i;

  brect = (Rect *)myalloc(sizeof(Rect)*INITBRECT);
  
  for(i=0;i<INITBRECT;i++){
    brect[i].xs=NORECT;
  }
}

void init_cb_tab()
{
  int i;

  cb_tab = (Label *)myalloc(sizeof(Label)*LABELnbr);
  for(i=0;i<LABELnbr;i++)
    cb_tab[i]=NOCB;
}

void resize_brect()
{
  int i;

  brect = (Rect *)myrealloc(brect,current_brect_size,INCBRECT,sizeof(Rect));
  for(i=current_brect_size;i<current_brect_size+INCBRECT;i++){
    brect[i].xs=NORECT;
  }
  current_brect_size+=INCBRECT;
}

void set_brect(Label lab1, Label lab2)
{
  int i;
  Label removed;

  if(cb_tab[lab1]==NOCB && cb_tab[lab2]==NOCB){
    /* if both of the connected components with the labels lab1 and lab2
       are not registered in a segmented block */
    brect[BRECTid]=merge_rect(crect[lab1], crect[lab2]);
    cb_tab[lab1]=BRECTid;
    cb_tab[lab2]=BRECTid;
    BRECTid++;
  }
  else if(cb_tab[lab1]!=NOCB && cb_tab[lab2]==NOCB){
    /* if only the connected component with the label lab1
       is registered in a segmented block with id cb_tab[lab1] */
    brect[cb_tab[lab1]]=merge_rect(brect[cb_tab[lab1]],crect[lab2]);
    cb_tab[lab2]=cb_tab[lab1];
  }
  else if(cb_tab[lab1]==NOCB && cb_tab[lab2]!=NOCB){
    /* vice versa */
    brect[cb_tab[lab2]]=merge_rect(crect[lab1],brect[cb_tab[lab2]]);
    cb_tab[lab1]=cb_tab[lab2];
  }
  else {/* cb_tab[lab1]!=NOCB && cb_tab[lab2]!=NOCB 
	   if both of the connected components have already been 
	   registered */
    if(cb_tab[lab1]!=cb_tab[lab2]) { /* already registered as different
					blocks */
      /* merge them */
      removed=cb_tab[lab2];
      brect[cb_tab[lab1]]=merge_rect(brect[cb_tab[lab1]],
				     brect[removed]);
      brect[removed].xs=NORECT;
      for(i=0;i<LABELnbr;i++)
	if(cb_tab[i]==removed) cb_tab[i]=cb_tab[lab1];
    }
    /* otherwise, just ignore */
  }
}

Rect merge_rect(Rect r1, Rect r2)
{
  Rect merged;

  merged.xs = (((r1.xs)<(r2.xs))?(r1.xs):(r2.xs));
  merged.xe = (((r1.xe)>(r2.xe))?(r1.xe):(r2.xe));
  merged.ys = (((r1.ys)<(r2.ys))?(r1.ys):(r2.ys));
  merged.ye = (((r1.ye)>(r2.ye))?(r1.ye):(r2.ye));

  return merged;
}

int is_noise_brect(Rect rect)
{
  /* Add some tests for a block bounding box 
     for example .... */

  /* definition of noise blocks */
  if(rarea(rect) < brect_area_min){ /* small enough */
    return YES;
  }
  else if(rwidth(rect) < rheight(rect) && /* vertical */
     raspect(rect) > brect_aspect_max && /* elongated */
     rarea(rect) < brect_varea_min){ /* small enough for not columns */
    /* only for documents with western languages
       assumption : text-lines are horizontal */
    return YES;
  }
  else{
    return NO;
  }
}
#endif /* RECT */
