#include "pset.h"

void err(char *message, int contin){
  char *constr, *errstr;
  if (contin>=EXIT) constr="Exiting.";
  else constr="Continuing...";
  if (contin>WARN) errstr="ERROR";
  else errstr="WARNING";
  fprintf(stderr, "#%s: %s.  %s\n", errstr, message, constr);
  if (contin==USAGE) Usage();
  if (contin>=EXIT) exit(EXIT_FAILURE);
  fflush(stderr);
}

void *Alloc(size_t Size){
  void *p;
  if (!(p = malloc(Size))) err("could not allocate memory", EXIT);
  return(p);
}

FILE *fOpen(const char *filename, const char *mode){
  FILE *f;
  char *errstr;
  if (strcmp(filename,"") * strcmp(filename,"-")) {
    f=fopen(filename, mode);
    if (f) return(f);
    else {
      errstr=new(char, 272); /*max unix filename is 256, plus "unable to..."*/
      sprintf(errstr, "unable to open %s", filename);
      err(errstr, USAGE);
    }
  }
  else
    return(mode[0]=='r'?stdin:stdout);
  return(NULL);
}

void fClose(FILE *f){
  if ((f != stdin) && (f != stdout))
    fclose(f);
}

int sh_c(const char *command, 
	 char *standardout, const int standardoutmax, 
	 char *standarderr, const int standarderrmax){
  /* this is intended to act like `sh -c command >standardout 2> standarderr` 

     sh will be asked to perform action.  
     at most standardoutmax characters of standard output will be assigned
        to the string standardout, which must be able to contain them.
     at most standarderrormax characters of standard error output from the 
        command will similarly be assigned to standarderror, which also must 
	be previously allocated.
     returns the exit status of the command.
     
     NOTE: as a general rule, this function is slow.
     NOTE: check this with 'assert(!sh_c(...));' as most functions, returning 
     success, 0, will raise an assertion failed. 
     */
  int resultcode;  /* stores the return code while retrieving output */
  static char *standardouttmp=NULL, *standarderrtmp=NULL; 
      /* temp files that output gets redirected to (static to save time) */
  static char *cmdline=NULL; /* holds the modified command string */
  int endchar; /* allows me to null-terminate the string properly */
  FILE *tmp;
  
  if (!cmdline) { /* this is the first time sh_c is called */
    cmdline = new(char, strlen(command) + 100);
    standardouttmp = new(char, PATH_MAX);
    standarderrtmp = new(char, PATH_MAX);

    strcpy(standardouttmp, tmpnam(NULL));
    strcpy(standarderrtmp, tmpnam(NULL));
  }
  
  sprintf(cmdline, "(%s) 1> %s 2>%s", command, standardouttmp, standarderrtmp);
  
  resultcode = system(cmdline);
  
  tmp = fOpen(standardouttmp, "r");
  endchar=(int) fread(standardout, sizeof(char), (size_t) standardoutmax, tmp);
  fClose(tmp);
  standardout[endchar]='\0';
  /* remove(standardouttmp); *no longer removing, takes too much time. 
   *                         *instead I will just leave these files around
   *                         *as trash. */
  
  tmp = fOpen(standarderrtmp, "r");
  endchar=(int) fread(standarderr, sizeof(char), (size_t) standarderrmax, tmp);
  fClose(tmp);
  standarderr[endchar]='\0';
  /* remove(standarderrtmp); * see above */
  
  /* free(cmdline);
     free(standardouttmp);
     free(standarderrtmp); * see above */
  return resultcode;
}









