/*  
**  File: Docstrum.c
**  Purpose: Docstrum segmentation algorithm implementation
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  References: The Document Spectrum for Page Layout Analysis ,
**              IEEE PAMI Vol. 15, No. 11, Nov. 1993 pp. 1162-1173
**  $Id: Docstrum.c,v 1.1 2000/09/19 22:20:48 maosong Exp maosong $
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#include "pset.h"
void ParseArgDocstrum ( int argc, char **argv, DOCSTRUM_PARAM **para_p);

int main(int argc, char **argv)
{
	DOCSTRUM_PARAM *para;

	para = (DOCSTRUM_PARAM *) malloc (sizeof (DOCSTRUM_PARAM));

	ParseArgDocstrum(argc, argv, &para);

	DocstrumFunction(argv[9], argv[10], para, argv[11]);
	
	free(para);

  	return(0);

} 

/* < Parsing the Arguments */
void ParseArgDocstrum ( int argc, char **argv, DOCSTRUM_PARAM **para_p) {
	int c;
	int errflg=0;

	(*para_p)->knn = 8;
	(*para_p)->lsize = 2;
	(*para_p)->hsize = 500;
	(*para_p)->hatol = 30;
	(*para_p)->vatol = 30;
	(*para_p)->scr = 0.4;

	while((c = getopt(argc, argv, "t:p:u:d:")) != EOF)
        	switch (c) {
        		case 't':
                		sscanf(optarg,"%f", &((*para_p)->tcc));
                		break;
        		case 'p':
                		sscanf(optarg,"%f", &((*para_p)->par));
                		break;
        		case 'u':
                		sscanf(optarg,"%f", &((*para_p)->per));
                		break;
        		case 'd':
                		sscanf(optarg,"%f", &((*para_p)->dis));
                		break;
        		case '?':
                		errflg++;
                		break;

        	}

	if(argc==1) {
  		printf("\nUsage: Docstrum -t <val> -p <val> -u <val> -d <val> <img> <cc_file> <seg_file>\n\n");
  		printf("-t val:   textline threshold factor(in pixels)\n");
  		printf("-p val:   parallel threshold factor( in pixels)\n");
  		printf("-u val:   perpendicular threshold factor(in pixels)\n");
  		printf("-d val:   component size factor\n");
  		printf("img: input image file name (TIFF format)\n");
  		printf("cc_file:   stores connected component coordinates\n");
  		printf("seg_file: output segmentation result file name (DAFS format)\n\n");
  		exit(1);
	}
	if(errflg!=0 || argc != 12) {
  		printf("Wrong argument(s)!\n");
  		exit(1);
	}

}

/* > */

