/*  
**  File: TestSeg.c
**  Purpose: main program: read parameters for automatically train
**	     a practical objective function using various optimization algorithms
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: TestSeg.c,v 1.1 2000/09/19 22:20:48 maosong Exp maosong $
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "pset.h"

int main(int argc, char **argv)
{ 
	TEST_PROTOCOL	*test_protocol;
	char		*test_report;
	char		*test_protocol_file;
	char		*seg_file;
	char		*ben_file;
	char		*weight_file;

	void		*ben_param;
	void		*seg_param;

  	ParseArgTest(argc, argv, &test_report, &test_protocol_file,
			&seg_file, &ben_file, &weight_file);

        ReadTestProtocol(test_protocol_file, &test_protocol); 	/* read training protocol */

        switch(test_protocol->ben_alg) {
                case TEXTLINE_BASED:
                        ReadBenchParam(ben_file, &ben_param);  /* read benchmarking algorithm parameters */
                        break;
                default:
                        printf("No such benchmarking algorithm.\n");
                        exit(-1);
                        break;
        }

        switch(test_protocol->seg_alg) {
                case XYCUT:
                        ReadXycutParam(seg_file, &seg_param);	/* read X-Y cut algorithm parameters */
                        break;
                case DOCSTRUM:
                        ReadDocstrumParam(seg_file, &seg_param); /* read Docstrum algorithm parameters */
                        break;
                case VORONOI:
                        ReadVoronoiParam(seg_file, &seg_param);	/* read Docstrum algorithm parameters */
                        break;
                case TEST_DRIVER:
                        break;
                default:
                        printf("No such segmentation algorithm.\n");
                        exit(-1);
                        break;
        }

	WriteTestReportHeader(argc, argv, test_protocol, &test_report);

	Test(test_protocol, ben_param, seg_param, weight_file, &test_report);	/* train algorithm function */

	FreeJobTest(test_protocol, test_report, test_protocol_file,    /* free allocated memory */
		     ben_param, seg_param, seg_file, ben_file, weight_file);
	return(0);
}

