/*  
**  File: TrainSeg.c
**  Purpose: main program: read parameters for automatically train
**	     a practical objective function using various optimization algorithms
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   22 March 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  $Id: TrainSeg.c,v 1.1 2000/09/19 22:20:48 maosong Exp maosong $
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "pset.h"

int main(int argc, char **argv)
{ 
	TRAIN_PROTOCOL	*train_protocol;
	char		*train_report;
	char		*opt_seg_param;
	char		*train_protocol_file;
	char		*opt_file;
	char		*seg_file;
	char		*ben_file;
	char		*weight_file;

	void		*opt_param;
	void		*ben_param;
	void		*seg_param;

  	ParseArgTrain(argc, argv, &train_report, &opt_seg_param, &train_protocol_file,
			&opt_file, &seg_file, &ben_file, &weight_file);

        ReadTrainProtocol(train_protocol_file, &train_protocol); /* read training protocol */

        switch(train_protocol->opt_alg) {
                case SIMPLEX:
                        ReadSimplexParam(opt_file, &opt_param);	/* read Nelder-Mead simplex algorithm parameters */
                        break;
                default:
                        printf("No such optimization algorithm.\n");
                        exit(-1);
                        break;
        }

        switch(train_protocol->ben_alg) {
                case TEXTLINE_BASED:
                        ReadBenchParam(ben_file, &ben_param);	/* read benchmarking algorithm parameters */
                        break;
                default:
                        printf("No such benchmarking algorithm.\n");
                        exit(-1);
                        break;
        }

        switch(train_protocol->seg_alg) {
                case XYCUT:
                        ReadXycutParam(seg_file, &seg_param);	/* read X-Y cut algorithm parameters */
                        break;
                case DOCSTRUM:
                        ReadDocstrumParam(seg_file, &seg_param); /* read Docstrum algorithm parameters */
                        break;
                case VORONOI:
                        ReadVoronoiParam(seg_file, &seg_param); /* read Docstrum algorithm parameters */
                        break;
                case TEST_DRIVER:
                        break;
                default:
                        printf("No such segmentation algorithm.\n");
                        exit(-1);
                        break;
        }

	WriteTrainReportHeader(argc, argv, train_protocol, &train_report); 

	Train(train_protocol, opt_param, ben_param, seg_param, weight_file,
	      &train_report, &opt_seg_param);	/* train algorithm function */

	FreeJobTrain(train_protocol, train_report, opt_seg_param, 
		     train_protocol_file, opt_param, 
		     ben_param, seg_param, opt_file, seg_file,
		     ben_file, weight_file);
	return(0);
}

