/*  
**  File: Voronoi.c
**  Purpose: the Voronoi-based segmentation algorithm  implementation
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   18 January 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  References:
**
**  $Id: Voronoi.c,v 1.1 2000/09/19 22:20:48 maosong Exp maosong $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"
void ParseArgVoronoi(int argc, char **argv, VORONOI_PARAM **para_p);

int main(int argc, char **argv)
{
	VORONOI_PARAM *para;

	para = (VORONOI_PARAM *) malloc (sizeof(VORONOI_PARAM));

	ParseArgVoronoi(argc, argv, &para);

	VoronoiFunction(argv[9], para, argv[10]);

	free(para);

	return(0);
}

void ParseArgVoronoi(int argc, char **argv, VORONOI_PARAM **para_p)
{
        int c;
        int errflg = 0;
        extern char *optarg;

        while((c = getopt(argc, argv, "s:n:f:t:")) != EOF) {
                switch (c) {
			case 's':
                                sscanf(optarg,"%d", &((*para_p)->sr));
                                break;
                        case 'n':
                                sscanf(optarg,"%d", &((*para_p)->nm));
                                break;
                        case 'f':
                                sscanf(optarg,"%f", &((*para_p)->fr));
                                break;
                        case 't':
                                sscanf(optarg,"%d", &((*para_p)->ta));
                                break;
                        case '?':
                                errflg++;
                                break;

                }
        }
	
	(*para_p)->alg_mode = SHELL_CALL;
	(*para_p)->sw = 2;
	(*para_p)->Cw = 500;
	(*para_p)->Ch = 500;
	(*para_p)->Cr = 5;
	(*para_p)->Az = 50;
	(*para_p)->Al = 40000;
	(*para_p)->Br = 4;

        if(argc == 1) {
                printf("\nUsage: Voronoi -s <val> -n <val> -f <val> -t <val> <img> <seg_file>\n");
                printf("-s <val>: sampling rate\n");
                printf("-n <val>: maximum size threshold of noise connected
			component\n");
                printf("-f <val>: Margin control factor for Td2\n");
                printf("-t <val>: Area ratio threshold\n");
                printf("img: image input\n");
                printf("outdafs: dafs output.\n");
                exit(-1);
        }
        if(errflg != 0 || argc != 11) {
                printf("Wrong argument(s).\n");
                exit(-1);
        }
}

