/*  
**  File: Xycut.c
**  Purpose: X-Ycut segmentation algorithm  implementation
**  Author: Song Mao (maosong@cfar.umd.edu)
**  Date:   18 January 1999
**  Organization: Language and Media Processing Lab
**                 Center for Automation Research
**                   University of Maryland
**                   College Park, MD 20742
**  References:
**
**  $Id: Xycut.c,v 1.1 2000/09/19 22:20:48 maosong Exp maosong $
*/

#include <stdio.h>
#include <stdlib.h>

#include "pset.h"
void ParseArgXycut(int argc, char **argv, XYCUT_PARAM **para_p);

int main(int argc, char **argv)
{
	XYCUT_PARAM *para;

	para = (XYCUT_PARAM *) malloc (sizeof(XYCUT_PARAM));

	ParseArgXycut(argc, argv, &para);

	XycutFunction(argv[5], para, argv[6]);

	free(para);

	return(0);
}

void ParseArgXycut(int argc, char **argv, XYCUT_PARAM **para_p)
{
        int c;
        int errflg = 0;
        extern char *optarg;

        while((c = getopt(argc, argv, "a:c:")) != EOF) {
                switch (c) {
                        case 'a':
                                sscanf(optarg, "%d,%d",
                                        &((*para_p)->Tnx),
                                        &((*para_p)->Tny));
                                break;
                        case 'c':
                                sscanf(optarg,"%d,%d",
                                        &((*para_p)->Tcx),
                                        &((*para_p)->Tcy));
                                break;
                        case '?':
                                errflg++;
                                break;
                }
        }
        if(argc == 1) {
                printf("\nUsage: Xycut -a <val,val> -c <val,val> <img> <seg_file>\n");
                printf("-a val,val: noise removal thresholds in
                        X and Y directions.\n");
                printf("-c val,val: cut threshold thresholds in
                        X and Y directions.\n");
                printf("img: image input\n");
                printf("outdafs: dafs output.\n");
                exit(-1);
        }
        if(errflg != 0 || argc != 7) {
                printf("Wrong argument(s).\n");
                exit(-1);
        }
}

